#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.ComponentModel;
using System.Resources;
using System.Globalization;
using System.Diagnostics;
using System.Collections;
using System.Threading;
using Syncfusion.Windows.Forms;

namespace Syncfusion.GridHelperClasses
{
    /// <summary>
    ///    SR provides localized access to string resources specific 
    ///    from the assembly manifest Syncfusion.GridHelperClasses.Windows.SR.resources
    /// </summary>
    sealed class SR
    {
        #region Fields
        private ResourceManager resources;
        private static SR loader = null;
        #endregion

        #region Strings
        internal const string StartsWith = "StartsWith";
        internal const string EndsWith = "EndsWith";
        internal const string Equal = "Equals";
        internal const string NotEquals = "NotEquals";
        internal const string LessThan = "LessThan";
        internal const string LessThanOrEqualTo = "LessThanOrEqualTo";
        internal const string GreaterThan = "GreaterThan";
        internal const string GreaterThanOrEqualTo = "GreaterThanOrEqualTo";
        internal const string Like = "Like";
        internal const string Match = "Match";
        internal const string ExpressionMATCH = "ExpressionMATCH";
        internal const string All = "All";
        internal const string Custom = "Custom";
        internal const string Empty = "Empty";
        internal const string None = "None";
        internal const string FieldDialogBox = "FieldDialogBox";
        internal const string FieldTreeDialogBox = "FieldTreeDialogBox";
        internal const string SelectAll = "SelectAll";
        internal const string SortAtoZ = "SortAtoZ";
        internal const string SortZtoA = "SortZtoA";
        internal const string TextFilters="TextFilters";
        internal const string ClearFilterFrom = "ClearFilterFrom";
        internal const string Office2007FilterOK = "Office2007FilterOK";
        internal const string Office2007FilterCancel = "Office2007FilterCancel";
        internal const string showstherowswhere = "showstherowswhere";
        internal const string CustomAutoFilter = "CustomAutoFilter";        
        internal const string CustomAutoFilterEqual = "CustomAutoFilterEqual";
        internal const string CustomAutoFilterNotequal = "CustomAutoFilterNotequal";
        internal const string CustomAutoFilterGreaterthan = "CustomAutoFilterGreaterthan";
        internal const string CustomAutoFilterGreaterthanOrEqual = "CustomAutoFilterGreaterthanOrEqual";
        internal const string CustomAutoFilterLessthan = "CustomAutoFilterLessthan";
        internal const string CustomAutoFilterlessthanOrEqual = "CustomAutoFilterlessthanOrEqual";
        internal const string CustomAutoFilterLike = "CustomAutoFilterLike";
        internal const string CustomAutoFilterMatch = "CustomAutoFilterMatch";
        internal const string CustomAutoFilterBeginsWith = "CustomAutoFilterBeginsWith";
        internal const string CustomAutoFilterEndsWith = "CustomAutoFilterEndsWith";
        internal const string CustomAutoFilterCancel = "CustomAutoFilterCancel";
        internal const string CustomAutoFilterOK = "CustomAutoFilterOK";
        internal const string CustomAutoFilterContains = "CustomAutoFilterContains";
        internal const string Office2007FilterEquals = "Office2007FilterEquals";
        internal const string Office2007FilterNotEquals = "Office2007FilterNotEquals";
        internal const string Office2007FilterBeginswith = "Office2007FilterBeginswith";
        internal const string Office2007FilterEndswith = "Office2007FilterEndswith";
        internal const string Office2007FilterContains = "Office2007FilterContains";
        internal const string Office2007FilterCustomFilter = "Office2007FilterCustomFilter";
        internal const string Office2007FilterCustomFilteror = "Office2007FilterCustomFilteror";
        internal const string Office2007FilterCustomFilterand = "Office2007FilterCustomFilterand";
        internal const string Error = "Error";
        internal const string CustomComboboxAutoFilterEqual = "CustomComboboxAutoFilterEqual";
        internal const string CustomComboboxAutoFilterNotequal = "CustomComboboxAutoFilterNotequal";
        internal const string CustomComboboxAutoFilterGreaterthan = "CustomComboboxAutoFilterGreaterthan";
        internal const string CustomComboboxAutoFilterGreaterthanOrEqual = "CustomComboboxAutoFilterGreaterthanOrEqual";
        internal const string CustomComboboxAutoFilterLessthan = "CustomComboboxAutoFilterLessthan";
        internal const string CustomComboboxAutoFilterlessthanOrEqual = "CustomComboboxAutoFilterlessthanOrEqual";
        internal const string CustomComboboxAutoFilterLike = "CustomComboboxAutoFilterLike";
        internal const string CustomComboboxAutoFilterMatch = "CustomComboboxAutoFilterMatch";
        internal const string CustomComboboxAutoFilterBeginsWith = "CustomComboboxAutoFilterBeginsWith";
        internal const string CustomComboboxAutoFilterEndsWith = "CustomComboboxAutoFilterEndsWith";
        internal const string CustomAutoFilterDialogBox="CustomAutoFilterDialogBox";
        #endregion

        #region Constructor and Destructor
        private SR()
        {
            this.resources = new ResourceManager(this.GetType());
        }

        public static void ReleaseResources()
        {
            if (SR.loader != null)
                SR.loader.resources.ReleaseAllResources();
        }
        #endregion

        #region Implementations
        private static SR GetLoader()
        {
            lock (typeof(SR))
            {
                if (SR.loader == null)
                    SR.loader = new SR();
                return SR.loader;
            }
        }
        static bool isGetStringError = false;
        // Methods

        public static string GetString(CultureInfo culture, string name, params object[] args)
        {
            if (isGetStringError)
                return name;
            SR sr = SR.GetLoader();
            string value = string.Empty;

            if (sr == null)
            {
                return null;
            }

            try
            {
                if (LocalizationProvider.Provider != null)
                {
                    String result = string.Empty;

                    result = LocalizationProvider.Provider.GetLocalizedString(culture, name, args);

                    if (result != string.Empty)
                        return result;
                }

                if (name != null)
                    value = sr.resources.GetString(name, culture);
                if (value != null && args != null && args.Length > 0)
                {
                    return String.Format(value, args);
                }

                return value;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                //TraceUtil.TraceExceptionCatched(ex);
                //if (!ExceptionManager.RaiseExceptionCatched(null, ex))
                //{
                //    throw;
                //}
                isGetStringError = true;
                return name;
            }
        }


        public static string GetString(string name)
        {
            return SR.GetString(null, name);
        }


        public static string GetString(string name, params object[] args)
        {
            return SR.GetString(null, name, args);
        }


        public static string GetString(CultureInfo culture, string name)
        {
            SR sr = SR.GetLoader();
            if (sr == null)
                return null;
            try
            {
                if (LocalizationProvider.Provider != null)
                {
                    String result = string.Empty;

                    result = LocalizationProvider.Provider.GetLocalizedString(culture, name, null);

                    if (result != string.Empty)
                        return result;
                }
                return sr.resources.GetString(name, culture);
            }
            catch
            {
                return name;
            }
        }

        private string GetFilterOptions(string name)
        {            
            return name;
        }

        public static object GetObject(CultureInfo culture, string name)
        {
            SR sr = SR.GetLoader();
            if (sr == null)
                return null;
            return sr.resources.GetObject(name, culture);
        }


        public static object GetObject(string name)
        {
            return SR.GetObject(null, name);
        }



        public static bool GetBoolean(CultureInfo culture, string name)
        {
            bool value;
            SR sr = SR.GetLoader();
            object obj;
            value = false;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Boolean)
                    value = ((bool)obj);
            }
            return value;
        }


        public static bool GetBoolean(string name)
        {
            return SR.GetBoolean(name);
        }


        public static byte GetByte(CultureInfo culture, string name)
        {
            byte value;
            SR sr = SR.GetLoader();
            object obj;
            value = (byte)0;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Byte)
                    value = ((byte)obj);
            }
            return value;
        }


        public static byte GetByte(string name)
        {
            return SR.GetByte(null, name);
        }


        public static char GetChar(CultureInfo culture, string name)
        {
            char value;
            SR sr = SR.GetLoader();
            object obj;
            value = (char)0;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Char)
                    value = (char)obj;
            }
            return value;
        }


        public static char GetChar(string name)
        {
            return SR.GetChar(null, name);
        }


        public static double GetDouble(CultureInfo culture, string name)
        {
            double value;
            SR sr = SR.GetLoader();
            object obj;
            value = 0.0;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Double)
                    value = ((double)obj);
            }
            return value;
        }


        public static double GetDouble(string name)
        {
            return SR.GetDouble(null, name);
        }


        public static float GetFloat(CultureInfo culture, string name)
        {
            float value;
            SR sr = SR.GetLoader();
            object obj;
            value = 0.0f;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Single)
                    value = ((float)obj);
            }
            return value;
        }


        public static float GetFloat(string name)
        {
            return SR.GetFloat(null, name);
        }


        public static int GetInt(string name)
        {
            return SR.GetInt(null, name);
        }


        public static int GetInt(CultureInfo culture, string name)
        {
            int value;
            SR sr = SR.GetLoader();
            object obj;
            value = 0;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Int32)
                    value = ((int)obj);
            }
            return value;
        }


        public static long GetLong(string name)
        {
            return SR.GetLong(null, name);
        }


        public static long GetLong(CultureInfo culture, string name)
        {
            Int64 value;
            SR sr = SR.GetLoader();
            object obj;
            value = ((Int64)0);
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Int64)
                    value = ((Int64)obj);
            }
            return value;
        }

        public static short GetShort(CultureInfo culture, string name)
        {
            short value;
            SR sr = SR.GetLoader();
            object obj;
            value = (short)0;
            if (sr != null)
            {
                obj = sr.resources.GetObject(name, culture);
                if (obj is System.Int16)
                    value = ((short)obj);
            }
            return value;
        }


        public static short GetShort(string name)
        {
            return SR.GetShort(null, name);
        }
        #endregion
    }

    /// <summary>
    /// Specifies the category in which the property or event will be displayed in a visual designer.
    /// </summary>
    /// <remarks>
    /// This is a localized version of CategoryAttribute. The localized string will be loaded from the 
    /// assembly manifest Syncfusion.GridHelperClasses.Windows.SR.resources
    /// </remarks>
    [AttributeUsage(AttributeTargets.All, AllowMultiple = false, Inherited = true)]
    sealed class SRCategoryAttribute : CategoryAttribute
    {
        public SRCategoryAttribute(string category)
            : base(category)
        {
        }

        protected override string GetLocalizedString(string value)
        {
            return SR.GetString(value);
        }
    }

    /// <summary>
    /// Specifies a description for a property or event.
    /// </summary>
    /// <remarks>
    /// This is a localized version of DescriptionAttribute. The localized string will be loaded from the 
    /// assembly manifest Syncfusion.GridHelperClasses.Windows.SR.resources
    /// </remarks>
    [AttributeUsage(AttributeTargets.All, AllowMultiple = false, Inherited = true)]
    sealed class SRDescriptionAttribute : DescriptionAttribute
    {
        private bool replaced = false;

        public SRDescriptionAttribute(string description)
            : base(description)
        {
        }

        public override string Description
        {
            get
            {
                if (!this.replaced)
                {
                    this.replaced = true;
                    this.DescriptionValue = SR.GetString(base.Description);
                }
                return base.Description;
            }
        }
    }
    /// <summary>
    /// DynamicFilterResourceIdentifiers contains Ids specific to the Syncfusion.GridHelperClasses.Windows namespace. 
    /// </summary>
    public sealed class DynamicFilterResourceIdentifiers
    {
        #region Strings Constants
        public const string StartsWith = "StartsWith";
        public const string EndsWith = "EndsWith";
        public const string Equal = "Equals";
        public const string NotEquals = "NotEquals";
        public const string LessThan = "LessThan";
        public const string LessThanOrEqualTo = "LessThanOrEqualTo";
        public const string GreaterThan = "GreaterThan";
        public const string GreaterThanOrEqualTo = "GreaterThanOrEqualTo";
        public const string Like = "Like";
        public const string Match = "Match";
        public const string ExpressionMATCH = "ExpressionMATCH";
        public const string All = "All";
        public const string Custom = "Custom";
        public const string Empty = "Empty";
        public const string None = "None";
        public const string FieldDialogBox = "FieldDialogBox";
        public const string FieldTreeDialogBox = "FieldTreeDialogBox";
        public const string SelectAll= "SelectAll";
        public const string SortAtoZ = "SortAtoZ";
        public const string SortZtoA = "SortZtoA";
        public const string TextFilters = "TextFilters";
        public const string ClearFilterFrom = "ClearFilterFrom";
        public const string Office2007FilterOK = "Office2007FilterOK";
        public const string Office2007FilterCancel = "Office2007FilterCancel";
        public const string showstherowswhere = "showstherowswhere";
        public const string CustomAutoFilter = "CustomAutoFilter";        
        public const string CustomAutoFilterEqual="CustomAutoFilterEqual";
        public const string CustomAutoFilterNotequal="CustomAutoFilterNotequal";
        public const string CustomAutoFilterGreaterthan="CustomAutoFilterGreaterthan";
        public const string CustomAutoFilterGreaterthanOrEqual="CustomAutoFilterGreaterthanOrEqual";
        public const string CustomAutoFilterLessthan="CustomAutoFilterLessthan";
        public const string CustomAutoFilterlessthanOrEqual="CustomAutoFilterlessthanOrEqual";
        public const string CustomAutoFilterLike="CustomAutoFilterLike";
        public const string CustomAutoFilterMatch="CustomAutoFilterMatch";
        public const string CustomAutoFilterBeginsWith="CustomAutoFilterBeginsWith";
        public const string CustomAutoFilterEndsWith = "CustomAutoFilterEndsWith";
        public const string CustomAutoFilterCancel = "CustomAutoFilterCancel";
        public const string CustomAutoFilterContains = "CustomAutoFilterContains";
        public const string CustomAutoFilterOK = "CustomAutoFilterOK";
        public const string Office2007FilterEquals="Office2007FilterEquals";
        public const string Office2007FilterNotEquals="Office2007FilterNotEquals";
        public const string Office2007FilterBeginswith="Office2007FilterBeginswith";
        public const string Office2007FilterEndswith="Office2007FilterEndswith";
        public const string Office2007FilterContains="Office2007FilterContains";
        public const string Office2007FilterCustomFilter = "Office2007FilterCustomFilter";
        public const string Office2007FilterCustomFilteror = "Office2007FilterCustomFilteror";
        public const string Office2007FilterCustomFilterand = "Office2007FilterCustomFilterand";
        public const string Error = "Error";
        public const string CustomAutoFilterDialogBox = "CustomAutoFilterDialogBox";
        public const string CustomComboboxAutoFilterEqual = "CustomComboboxAutoFilterEqual";
        public const string CustomComboboxAutoFilterNotequal = "CustomComboboxAutoFilterNotequal";
        public const string CustomComboboxAutoFilterGreaterthan = "CustomComboboxAutoFilterGreaterthan";
        public const string CustomComboboxAutoFilterGreaterthanOrEqual = "CustomComboboxAutoFilterGreaterthanOrEqual";
        public const string CustomComboboxAutoFilterLessthan = "CustomComboboxAutoFilterLessthan";
        public const string CustomComboboxAutoFilterlessthanOrEqual = "CustomComboboxAutoFilterlessthanOrEqual";
        public const string CustomComboboxAutoFilterLike = "CustomComboboxAutoFilterLike";
        public const string CustomComboboxAutoFilterMatch = "CustomComboboxAutoFilterMatch";
        public const string CustomComboboxAutoFilterBeginsWith = "CustomComboboxAutoFilterBeginsWith";
        public const string CustomComboboxAutoFilterEndsWith = "CustomComboboxAutoFilterEndsWith";
       
        #endregion        
    }


} 