#region Copyright Syncfusion Inc. 2001 - 2009
// Copyright Syncfusion Inc. 2001 - 2009. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Data;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using Syncfusion.Grouping; 
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms;
using Syncfusion.Collections.BinaryTree;
using System.Collections.Generic;
using System.Drawing.Drawing2D;

namespace Syncfusion.GridHelperClasses
{
    /// <summary>
    /// Implements the DataModel part for a ExcelFilterCell.
    /// </summary>
	public class Grid2007ExcelFilterCellModel : GridHeaderCellModel
	{
        /// <summary>
        /// Initializes a new <see cref="Grid2007ExcelFilterCellModel"/> object
        /// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
        /// </summary>
        /// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>
        /// <remarks>
        /// You typically access cell models through the <see cref="GridModel.CellModels"/>
        /// property of the <see cref="GridModel"/> class.
        /// </remarks>
		public Grid2007ExcelFilterCellModel(GridModel grid)
			: base(grid)
		{
		}

		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new Grid2007ExcelFilterCellRenderer(control, this);
		}
	}
    /// <summary>
    /// Implements the renderer part of a ExcelFilterCell.
    /// </summary>
	public class Grid2007ExcelFilterCellRenderer: GridHeaderCellRenderer
	{
        private FilterDropDownControl ddUser;
		Rectangle filterRect = Rectangle.Empty;   
		bool IsMouseOver = false;
		int colindex = -1;		                	
        Hashtable filterImages;
        int filterRow=-1, Filtercol=-1;
        ToolStripMenuItem sortAsc;
        ToolStripMenuItem sortDesc;
        ToolStripMenuItem clearFilter;
        ToolStripMenuItem textFilters;
        ToolStripMenuItem equal;
        ToolStripMenuItem notEqual;
        ToolStripMenuItem beginsWith;
        ToolStripMenuItem endsWith;
        ToolStripMenuItem contains;        
        ToolStripMenuItem customFilter;
        
        ContextMenuStrip filterDialog;        
        string clearFilterString = SR.GetString(SR.ClearFilterFrom);
        int filterHeight = 15, filterWidth = 15;
        private string compareText1, compareText2;

        #region Constructor

        /// <summary>
        /// Initializes a new <see cref="Grid2007ExcelFilterCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridCellModelBase"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="Grid2007ExcelFilterCellModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>
        /// References to GridControlBase and GridCellModelBase will be saved.
        /// </remarks>
        public Grid2007ExcelFilterCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
            filterImages = new Hashtable();
            filterImages.Add("Filter_gray", DynamicFilterBitmaps.GetBitmap("filter_funnel_gr"));
            filterImages.Add("ClearFilter_gray", DynamicFilterBitmaps.GetBitmap("filter_set_gr"));
            filterImages.Add("Filter_white", DynamicFilterBitmaps.GetBitmap("filter_metro"));
            filterImages.Add("ClearFilter_white", DynamicFilterBitmaps.GetBitmap("filter_set_wh"));
            //filterImages.Add("RemoveFilter", );

            filterDialog = new ContextMenuStrip();

            this.ddUser = new FilterDropDownControl(this.Grid.GetGridVisualStylesDrawing().VisualStyle.ToString());
            //hook the usercontrol save and cancel events...
            this.ddUser.UserControlSave += new EventHandler(ddUser_UserControlSave);
            this.ddUser.UserControlCancel += new EventHandler(ddUser_UserControlCancel);
            this.ddUser.Dock = DockStyle.Fill;
            sortAsc = new ToolStripMenuItem(SR.GetString(SR.SortAtoZ), DynamicFilterBitmaps.GetBitmap("sortasc"), ItemSelection_Click);
            sortAsc.CheckOnClick = true;            
            this.filterDialog.Items.Add(sortAsc);
            sortDesc = new ToolStripMenuItem(SR.GetString(SR.SortZtoA), DynamicFilterBitmaps.GetBitmap("sortdesc"), ItemSelection_Click);
            sortDesc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortDesc);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            this.clearFilter = new ToolStripMenuItem(SR.GetString(SR.ClearFilterFrom), DynamicFilterBitmaps.GetBitmap("filter_delete"), ItemSelection_Click);            
            this.filterDialog.Items.Add(clearFilter);
            textFilters = new ToolStripMenuItem(SR.GetString(SR.TextFilters));
            

            equal = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEquals), null, CustomItemSelection_Click);
            notEqual = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterNotEquals), null, CustomItemSelection_Click);
            beginsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterBeginswith), null, CustomItemSelection_Click);
            endsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEndswith), null, CustomItemSelection_Click);
            contains = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterContains), null, CustomItemSelection_Click);
            customFilter = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterCustomFilter), null, CustomItemSelection_Click);
            textFilters.DropDownItems.Add(equal);
            textFilters.DropDownItems.Add(notEqual);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(beginsWith);
            textFilters.DropDownItems.Add(endsWith);            
            textFilters.DropDownItems.Add(contains);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(customFilter);
            this.filterDialog.Items.Add(textFilters);
            this.filterDialog.Items.Add(new ToolStripSeparator());                        
            ToolStripControlHost filterDropDownStrip = new ToolStripControlHost(this.ddUser);            
            this.filterDialog.Items.Add(filterDropDownStrip);
            this.ddUser.BackColor = filterDropDownStrip.BackColor;
			this.Grid.CellClick +=new GridCellClickEventHandler(Grid_CellClick);          
            this.filterDialog.Opening += new CancelEventHandler(filterDialog_Opening);
        }

        void CustomItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.textFilters.DropDownItems.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            object[] uniqueList = GetFilterBarChoices(sty.TableCellIdentity);
            if (index!=-1)
            {   
                if (this.Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                {
                    MetroCustomRowFilter textFilter = new MetroCustomRowFilter(uniqueList);
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        this.compareText1 = textFilter.compareText1;
                        this.compareText2 = textFilter.compareText2;
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }
                else
                {
                    CustomRowFilter textFilter = new CustomRowFilter(this.Grid.Model.Options.GridVisualStyles.ToString(),uniqueList);
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        this.compareText1 = textFilter.compareText1;
                        this.compareText2 = textFilter.compareText2;
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }            
            }
        }

        void ItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.filterDialog.Items.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes=sty.TableCellIdentity.Table.TableDescriptor;
            string columnName=sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);                    
            switch (index)
            {
                case 0:
                    this.sortDesc.Checked = false;
                    if (this.sortAsc.Checked)
                    {
                        tableDes.SortedColumns.Clear();
                        tableDes.SortedColumns.Add(columnName);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 1:
                    this.sortAsc.Checked = false;
                    if (this.sortDesc.Checked)
                    {
                        tableDes.SortedColumns.Clear();
                        tableDes.SortedColumns.Add(columnName,ListSortDirection.Descending);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 3:                    
                    sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                    break;                
            }

        }

        void filterDialog_Opening(object sender, CancelEventArgs e)
        {
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            if (tableDes.SortedColumns.Contains(columnName))
            {
                if (tableDes.SortedColumns[columnName].SortDirection == ListSortDirection.Ascending)
                {
                    sortAsc.Checked = true;
                    sortDesc.Checked = false;
                }
                else if (tableDes.SortedColumns[columnName].SortDirection == ListSortDirection.Descending)
                {
                    sortAsc.Checked = false;
                    sortDesc.Checked = true;
                }
            }
            else if (sortAsc.Checked || sortDesc.Checked)
            {
                sortAsc.Checked = false;
                sortDesc.Checked = false;
            }

        }

        #endregion

        #region Office2007Filter control events
        void ddUser_UserControlCancel(object sender, EventArgs e)
        {
            this.filterDialog.Hide();
        }

        void ddUser_UserControlSave(object sender, EventArgs e)
        {            
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow,Filtercol] as GridTableCellStyleInfo;        
            IList list = ddUser.GetValues();
            string filterName = GetFilterName(sty);            
            RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName);
            if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
            if (list.Count>0)
            {
                rfc.LogicalOperator =FilterLogicalOperator.Or;
                foreach (object s in list)
                {
                    FilterCondition con = new FilterCondition(FilterCompareOperator.Equals, s);
                     if (s.ToString() == "(Blanks)")
                    {
                        con.CompareText = string.Empty;
                        con.CompareValue = string.Empty;
                        rfc.Conditions.Add(con);
                    }
                    else	
                        rfc.Conditions.Add(con);
                }
                sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
            }            
            this.filterDialog.Hide();
        }

        #endregion

        #region CellOverriden Methods

        private string FindImageFile(string bitmapName)
        {
            string bitmappath = "";
            for (int n = 0; n < 10; n++)
            {
                if (System.IO.File.Exists(bitmapName))
                    bitmappath = bitmapName;

                bitmapName = @"..\" + bitmapName;
            }
            return bitmappath;
        }

		protected override void OnMouseHoverEnter(int rowIndex, int colIndex)
		{
			IsMouseOver = true;
			colindex =colIndex; 
			this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
			base.OnMouseHoverEnter (rowIndex, colIndex);
		}
		
		protected override void OnMouseHoverLeave(int rowIndex, int colIndex, EventArgs e)
		{
			IsMouseOver =  false;
			colindex =colIndex;
			this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
			base.OnMouseHoverLeave (rowIndex, colIndex, e);
		}

        protected override void OnDrawDisplayText(Graphics g, Rectangle textRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            object tag = style.Tag;
            if (Grid.PrintingMode || !(tag is ListSortDirection))
            {
                tag = null;
            }

            ListSortDirection listSortDirection = ListSortDirection.Ascending;
            int margin = filterWidth;
            if (tag != null)
            {
                listSortDirection = (ListSortDirection)tag;
                margin = filterWidth + 12;
            }

            bool isTextRightToLeft = Grid.SortIconPlacement == SortIconPlacement.Left;
            bool isTextTop = Grid.SortIconPlacement == SortIconPlacement.Top;
            if (isTextRightToLeft || Grid.IsRightToLeft() || style.RightToLeft == RightToLeft.Yes)
            {
                GridUtil.OffsetLeft(ref textRectangle, margin);
            }
            else if (isTextTop)
            {
                GridUtil.OffsetTop(ref textRectangle, 0);
            }
            else
            {
                textRectangle.Width -= margin;
            }

            base.OnDrawDisplayText(g, textRectangle, rowIndex, colIndex, style);

            #region Image Rendering

            Rectangle imageRect;
            if (Grid.IsRightToLeft() || isTextRightToLeft || style.RightToLeft == RightToLeft.Yes)
            {
                imageRect = new Rectangle(textRectangle.Left - filterWidth, textRectangle.Y + (textRectangle.Height / 6), filterWidth, filterHeight);
            }
            else
            {
                imageRect = new Rectangle(textRectangle.Right, textRectangle.Y + (textRectangle.Height / 6), filterWidth, filterHeight);
            }
            GridTableCellStyleInfo tablestyle = style as GridTableCellStyleInfo;
            Image imag_gray = (Image)filterImages["Filter_gray"];
            Image clImage_gray = (Image)filterImages["ClearFilter_gray"];
            Image imag_white = (Image)filterImages["Filter_white"];
            Image clImage_white = (Image)filterImages["ClearFilter_white"];
            if (tablestyle.TableCellIdentity.Column != null && tablestyle.TableCellIdentity.Column.AllowFilter == true)
            {
                GridTableDescriptor tableDesc = ((GridTableControl)this.Grid).TableDescriptor;
                bool isFiltered = false;
                if (tableDesc != null && tableDesc.RecordFilters.Count > 0
                    && tableDesc.RecordFilters.Contains(tablestyle.TableCellIdentity.Column.Name)
                    && GridOffice2007Filter.EnableFilteredColumnIcon)
                {
                    isFiltered = true;
                }
                if (GridOffice2007Filter.ShowOffice2007FilterOnMouseHover)
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = imageRect;
                    }
                    if (isFiltered && tableDesc.RecordFilters.Contains(tablestyle.TableCellIdentity.Column.Name))
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(clImage_white, imageRect);
                        else
                            g.DrawImage(clImage_gray, imageRect);
                        isFiltered = false;
                    }
                    else if (((IsMouseOver && colindex == colIndex) || (this.Grid.Model.SelectedRanges.AnyRangeIntersects(GridRangeInfo.Cell(rowIndex, colIndex)))))
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(imag_white, imageRect);
                        else
                            g.DrawImage(imag_gray, imageRect);
                    }
                    
                }
                else
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = imageRect;
                    }

                    if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                    {
                        if (!isFiltered)
                            g.DrawImage(imag_white, imageRect);
                        else
                            g.DrawImage(clImage_white, imageRect);
                    }
                    else
                        if (!isFiltered)
                            g.DrawImage(imag_gray, imageRect);
                        else
                            g.DrawImage(clImage_gray, imageRect);

                }
            }

            #endregion
            if (tag != null)
            {
                string s = style.ValueMember; 
                int dig = (!string.IsNullOrEmpty(s) && s.Length > 1) ? 2 : 1;

                Rectangle rect;
                if (isTextRightToLeft)
                {
                    rect = new Rectangle(textRectangle.Left - margin, textRectangle.Y, 10, textRectangle.Height);
                }
                else if (isTextTop)
                {
                    rect = new Rectangle(textRectangle.X, textRectangle.Y - 8, textRectangle.Width, textRectangle.Height);
                }
                else
                {
                    rect = new Rectangle(textRectangle.Right + filterWidth, textRectangle.Y, 10, textRectangle.Height);
                }

                rect = GridUtil.CenterInRect(rect, new Size(8 * dig, 8));

                Brush brush = null;
                Pen pen1 = null;
                this.Grid.Model.Options.GridVisualStylesDrawing.GetSortIconBrush(out brush, out pen1);
                if (s != string.Empty)
                {
                    //// Draw sort-position slightly to the right and above.
                    rect.Offset(-4 * (dig - 1), -4);
                    Font numFont = new Font(style.Font.Facename, 6, style.Font.FontStyle);
                    if (isTextTop)
                        DrawText(g, s, numFont, new Rectangle(rect.X + 5, rect.Y + 5, rect.Width, rect.Height), style, pen1.Color, Grid.IsRightToLeft());
                    else
                        DrawText(g, s, numFont, rect, style, pen1.Color, Grid.IsRightToLeft());
                    numFont.Dispose();
                    rect.Offset(4 * (dig - 1), 4);

                    if (isTextRightToLeft || Grid.IsRightToLeft())
                    {
                        rect.Offset(6, 0);
                    }
                    else
                    {
                        rect.Offset(-6, 0);
                    }
                }

                int i2 = Math.Max(0, (rect.Height - 6) / 2);
                rect.Inflate(-i2, -i2);
                GridTriangleDirection triangleDirection = listSortDirection == ListSortDirection.Ascending ? GridTriangleDirection.Up : GridTriangleDirection.Down;
                GridPaintTriangle.Paint(g, rect, triangleDirection, brush, pen1, true);
                pen1.Dispose();
                brush.Dispose();
            }
		}

        protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
            Color clrBottom = Color.Empty;
            Color clrRight = Color.Empty;
            Color clrInteriorFirst = Color.Empty;
            Color clrInteriorLast = Color.Empty;

            if (this.Grid.Model.Options.GridVisualStylesDrawing.GetHeaderBorderColors(out clrBottom, out clrRight, out clrInteriorFirst, out clrInteriorLast) &&
               ((style.Themed && XPThemes.IsThemedOS && XPThemes.IsThemeActive && this.Grid.ThemesEnabled)
                      || ((style.Themed && this.Grid.ThemesEnabled && ((this.Grid.Model.Options.GridVisualStyles != GridVisualStyles.SystemTheme))))))
            {
                Rectangle rect1 = clientRectangle;
                Pen br = new Pen(clrBottom);
                g.DrawLine(br, new PointF(rect1.X, rect1.Y), new PointF((rect1.X + rect1.Width), rect1.Y));
                br.Dispose();
            }
        }

		private void Grid_CellClick(object sender, GridCellClickEventArgs e)
		{	
			if( filterRect.IntersectsWith(new Rectangle(e.MouseEventArgs.X,e.MouseEventArgs.Y,0,0)))
			{                
                GridRangeInfo range = GridRangeInfo.Cell(e.RowIndex, e.ColIndex);
                Rectangle rec= this.Grid.RangeInfoToRectangle(range);
                Point loc = new Point(rec.Right, rec.Bottom);
				GridTableControl  grid = this.Grid as GridTableControl;                
                filterRow = e.RowIndex;
                Filtercol = e.ColIndex;                
                GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
                object[] items = GetFilterBarChoices(sty.TableCellIdentity);
                clearFilter.Text = clearFilterString +" "+ sty.TableCellIdentity.Column.HeaderText;
                
                string filterName = GetFilterName(sty);
                RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName);
                if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                    clearFilter.Enabled = true;
                else
                    clearFilter.Enabled = false;

                IList filteredValues = this.GetFilterTextList(sty);                                
                GridTableCellStyleInfo formatStyle=new GridTableCellStyleInfo();
                formatStyle.CellValueType = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.CellValueType;
                formatStyle.Format = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.Format;
                this.ddUser.SetItems(items, filteredValues, formatStyle, sty, clearFilter.Enabled);
                if (sty.GetActiveGridView() != null && sty.GetActiveGridView().GridOfficeScrollBars == Syncfusion.Windows.Forms.OfficeScrollBars.Metro)
                {
                    this.filterDialog.Font = new Font("Segoe UI", 8.25f);
                    this.filterDialog.ForeColor = Color.FromArgb(51, 51, 51);
                }
                this.filterDialog.Show(grid, loc);
				e.Cancel = true;				
			}
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Returns an array of category keys for this group and all parent groups which is used by FilterBarCells and FilterBarSummary
        /// to compare whether the conditions should be applied to this group.
        /// </summary>
        /// <param name="tableCellIdentity">Cell identifier.</param>
        /// <returns>An array of category keys.</returns>
        public object[] GetUniqueGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            return g.UniqueGroupId;
        }

        /// <summary>
        /// Returns the unique choices to be displayed in the filterbar cell.
        /// </summary>
        /// <param name="tableCellIdentity">Table cell identifier.</param>
        /// <returns>Filter bar choices.</returns>
        public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            GridGroupingControl groupGrid = ((GridEngine)tableCellIdentity.Table.Engine).GroupingControl;
            if (groupGrid.OptimizeFilterPerformance)
            {
                GridCurrentCell cc = tableCellIdentity.Table.TableDescriptor.Engine.TableControl.CurrentCell;
                if (cc.Renderer is GridNestedTableControlCellRenderer)
                {
                    GridNestedTableControlCellRenderer rend = cc.Renderer as GridNestedTableControlCellRenderer;
                    cc = rend.GetNestedCurrentCell();
                }
                cc.Lock();
                tableCellIdentity.Table.CurrentElement = tableCellIdentity.DisplayElement;
                tableCellIdentity.Table.CurrentRecordManager.Lock();
                SummaryDescriptor sd = new SummaryDescriptor(
                                tableCellIdentity.Column.Name + "FilterBarChoices",
                                tableCellIdentity.Column.MappingName,
                                new CreateSummaryDelegate(FilterBarChoicesSummary.CreateSummaryMethod));
                sd.IgnoreRecordFilterCriteria = true;
                tableCellIdentity.Table.TableDescriptor.Summaries.Add(sd);
                ITreeTableSummary[] summaries = tableCellIdentity.Table.GetSummaries();
                int summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(sd);
                FilterBarChoicesSummary filtersummary = (FilterBarChoicesSummary)summaries[summaryIndex];
                object[] values = filtersummary.Values;
                tableCellIdentity.Table.TableDescriptor.Summaries.Remove(sd);
                tableCellIdentity.Table.SummariesDirty = true;
                tableCellIdentity.Table.CurrentRecordManager.Unlock();
                cc.Unlock();
                Array.Sort(values, new ValueComparer());
                return values;
            }
            else
            {
                int summaryIndex = -1;
                ITreeTableSummary[] summaries = tableCellIdentity.DisplayElement.ParentGroup.GetSummaries(tableCellIdentity.Table);
                if (summaries.Length.Equals(0))
                {
                    SummaryDescriptor sd = new SummaryDescriptor(
                                tableCellIdentity.Column.Name + "FilterBarChoices",
                                tableCellIdentity.Column.MappingName,
                                new CreateSummaryDelegate(FilterBarChoicesSummary.CreateSummaryMethod));
                    sd.IgnoreRecordFilterCriteria = true;
                    tableCellIdentity.Table.TableDescriptor.Summaries.Add(sd);
                    summaries = tableCellIdentity.Table.GetSummaries();
                    summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(sd);
                    tableCellIdentity.Table.TableDescriptor.Summaries.Remove(sd);
                }
                else
                {
                    summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(tableCellIdentity.Table.TableDescriptor.Summaries[tableCellIdentity.Column.Name + "FilterBarChoices"]);
                }
                if (summaryIndex != -1)
                {
                    FilterBarChoicesSummary filterBarChoicesSummary = (FilterBarChoicesSummary)summaries[summaryIndex];
                    Array.Sort(filterBarChoicesSummary.Values, new ValueComparer());
                    return filterBarChoicesSummary.Values;
                }
                else
                {
                    object[] result = null;
                    GridGroupingControl gc = tableCellIdentity.Table.Engine.ParentControl;
                    GridQueryFilterBarChoicesEventArgs qe = new GridQueryFilterBarChoicesEventArgs(tableCellIdentity.Column, false, tableCellIdentity.DisplayElement);
                    gc.OnQueryFilterBarChoices(qe);
                    if (!qe.Cancel)
                    {
                        result = qe.UniqueFilterBarValues;
                    }
                    Array.Sort(result, new ValueComparer());
                    return result;
                }
            }
        }

        /// <summary>
        /// Determines the name of the filter based on the cell style information passed in.
        /// </summary>
        /// <param name="sty">cell style information.</param>
        /// <returns>Name of the filter.</returns>
        public string GetFilterName(GridTableCellStyleInfo sty)
        {
            object[] id = GetUniqueGroupId(sty.TableCellIdentity);
            string filterName = "";
            if (id == null || id.Length == 0)
            {
                filterName = sty.TableCellIdentity.Column.Name;
            }
            else
            {
                filterName = sty.TableCellIdentity.Column.Name + "@" + sty.TableCellIdentity.DisplayElement.ParentGroup.UniqueGroupIdsToString();
            }
            return filterName;
        }

        private IList GetFilterTextList(GridTableCellStyleInfo sty)
        {
            List<string> filterValus = new List<string>();
            filterValus.Clear();
            string filterName = GetFilterName(sty);
            if (!GridUtil.IsEmpty(filterName))
            {
                if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                {
                    RecordFilterDescriptor fil = sty.TableCellIdentity.Table.TableDescriptor.RecordFilters[filterName];
                    if (fil.Conditions.Count > 0)
                    {
                        foreach (FilterCondition con in fil.Conditions)
                        {
                            filterValus.Add(con.CompareText);
                        }
                    }
                    else if (!string.IsNullOrEmpty(fil.Expression))
                    {
                        filterValus.Add(this.compareText1);
                        filterValus.Add(this.compareText2);
                    }
                }
            }
            return filterValus;
        }
        #endregion
    }

    /// <summary>
    /// Implements the DataModel part for an optimized ExcelFilterCell.
    /// </summary>
    public class GridExcelFilterCellModel : GridHeaderCellModel
    {
        /// <summary>
        /// Initializes a new <see cref="GridExcelFilterCellModel"/> object
        /// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
        /// </summary>
        /// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>
        /// <remarks>
        /// You typically access cell models through the <see cref="GridModel.CellModels"/>
        /// property of the <see cref="GridModel"/> class.
        /// </remarks>
        public GridExcelFilterCellModel(GridModel grid)
            : base(grid)
        {

        }
        /// <summary>
        /// Creates the renderer for the cell model.
        /// </summary>
        /// <param name="control">grid</param>
        /// <returns>the renderer</returns>
        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new GridExcelFilterCellRenderer(control, this);
        }
    }
    /// <summary>
    /// Implements the renderer part of an optimized ExcelFilterCell.
    /// </summary>
    public class GridExcelFilterCellRenderer : GridHeaderCellRenderer
    {
        private OptimizedFilterDropDownControl ddUser;
        private Rectangle filterRect = Rectangle.Empty;
        private bool IsMouseOver = false;
        private int colindex = -1;

        private Hashtable filterImages;
        private int filterRow = -1, Filtercol = -1;
        private ToolStripMenuItem sortAsc;
        private ToolStripMenuItem sortDesc;
        private ToolStripMenuItem clearFilter;
        private ToolStripMenuItem textFilters;
        private ToolStripMenuItem equal;
        private ToolStripMenuItem notEqual;
        private ToolStripMenuItem beginsWith;
        private ToolStripMenuItem endsWith;
        private ToolStripMenuItem contains;
        private ToolStripMenuItem customFilter;
        private int filterHeight = 15, filterWidth = 15;
        private ContextMenuStrip filterDialog;
        private string clearFilterString = SR.GetString(SR.ClearFilterFrom);
        private Dictionary<string, object[]> filteredColumnCollection = new Dictionary<string, object[]>();
        internal bool isCurrentCellEdited = false;
        #region Constructor

        /// <summary>
        /// Initializes a new <see cref="GridExcelFilterCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridCellModelBase"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="Grid2007ExcelFilterCellModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>
        /// References to GridControlBase and GridCellModelBase will be saved.
        /// </remarks>
        public GridExcelFilterCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            filterImages = new Hashtable();
            filterImages.Add("Filter_gray", DynamicFilterBitmaps.GetBitmap("filter_funnel_gr"));
            filterImages.Add("ClearFilter_gray", DynamicFilterBitmaps.GetBitmap("filter_set_gr"));
            filterImages.Add("Filter_white", DynamicFilterBitmaps.GetBitmap("filter_metro"));
            filterImages.Add("ClearFilter_white", DynamicFilterBitmaps.GetBitmap("filter_set_wh"));

            filterDialog = new ContextMenuStrip();

            this.ddUser = new OptimizedFilterDropDownControl(grid.GetGridVisualStylesDrawing().VisualStyle.ToString());
            //hook the usercontrol save and cancel events...
            this.ddUser.UserControlSave += new EventHandler(ddUser_UserControlSave);
            this.ddUser.UserControlCancel += new EventHandler(ddUser_UserControlCancel);
            this.ddUser.Dock = DockStyle.Fill;
            sortAsc = new ToolStripMenuItem(SR.GetString(SR.SortAtoZ), DynamicFilterBitmaps.GetBitmap("sortasc"), ItemSelection_Click);
            sortAsc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortAsc);
            sortDesc = new ToolStripMenuItem(SR.GetString(SR.SortZtoA), DynamicFilterBitmaps.GetBitmap("sortdesc"), ItemSelection_Click);
            sortDesc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortDesc);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            this.clearFilter = new ToolStripMenuItem(SR.GetString(SR.ClearFilterFrom), DynamicFilterBitmaps.GetBitmap("filter_delete"), ItemSelection_Click);
            this.filterDialog.Items.Add(clearFilter);
            textFilters = new ToolStripMenuItem(SR.GetString(SR.TextFilters));

            equal = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEquals), null, CustomItemSelection_Click);
            notEqual = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterNotEquals), null, CustomItemSelection_Click);
            beginsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterBeginswith), null, CustomItemSelection_Click);
            endsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEndswith), null, CustomItemSelection_Click);
            contains = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterContains), null, CustomItemSelection_Click);
            customFilter = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterCustomFilter), null, CustomItemSelection_Click);
            textFilters.DropDownItems.Add(equal);
            textFilters.DropDownItems.Add(notEqual);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(beginsWith);
            textFilters.DropDownItems.Add(endsWith);
            textFilters.DropDownItems.Add(contains);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(customFilter);
            this.filterDialog.Items.Add(textFilters);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            ToolStripControlHost filterDropDownStrip = new ToolStripControlHost(this.ddUser);
            this.filterDialog.Items.Add(filterDropDownStrip);
            this.ddUser.BackColor = filterDropDownStrip.BackColor;
            this.Grid.CellClick += new GridCellClickEventHandler(Grid_CellClick);
            this.filterDialog.Opening += new CancelEventHandler(filterDialog_Opening);
            filterLists = new Dictionary<string, List<string>>();
            this.Grid.CurrentCellEditingComplete += new EventHandler(Grid_CurrentCellEditingComplete);
        }

        void Grid_CurrentCellEditingComplete(object sender, EventArgs e)
        {
            isCurrentCellEdited = true;
        }

        /// <summary>
        /// Performs filtering based on the checked items in the CheckedListBox.
        /// </summary>
        /// <param name="sender">Tool strip item</param>
        /// <param name="e">event data</param>
        private void CustomItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.textFilters.DropDownItems.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            object[] uniqueList = GetRecord(filterName, sty); //GetFilterBarChoices(sty.TableCellIdentity);
            if (index != -1)
            {
                if (this.Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                {
                    MetroCustomRowFilter textFilter = new MetroCustomRowFilter(uniqueList);
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        if (filterLists.ContainsKey(filterName))
                        {
                            filterLists.Remove(filterName);
                        }
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                        if (!filterLists.ContainsKey(filterName))
                        {
                            filterLists.Add(filterName, GetFilterChoicesInStringList(GetFilterBarChoices(sty.TableCellIdentity)));
                            filterLists[filterName].Sort();
                        }
                        if (filteredColumnCollection.ContainsKey(filterName))
                            filteredColumnCollection[filterName] = this.items;
                        else
                            filteredColumnCollection.Add(filterName, this.items);
                    }
                }
                else
                {
                    CustomRowFilter textFilter = new CustomRowFilter(this.Grid.Model.Options.GridVisualStyles.ToString(),uniqueList);
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        if (filterLists.ContainsKey(filterName))
                        {
                            filterLists.Remove(filterName);
                        }
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                        if (!filterLists.ContainsKey(filterName))
                        {
                            filterLists.Add(filterName, GetFilterChoicesInStringList(GetFilterBarChoices(sty.TableCellIdentity)));
                            filterLists[filterName].Sort();
                        }
                        if (filteredColumnCollection.ContainsKey(filterName))
                            filteredColumnCollection[filterName] = this.items;
                        else
                            filteredColumnCollection.Add(filterName, this.items);
                    }
                }                
            }
        }

        private void GenarateFilterColumnCollections(string filterName, GridTableCellStyleInfo sty)
        {
            List<string> filterItems = GetFilteredRecord(filterName, sty);
            List<string> filterListItems = null;
            filterItems.Sort();
            filterLists.TryGetValue(filterName, out filterListItems);
            if (filterListItems != null)
            {
                List<object> filterListTemp = new List<object>();
                filterListItems.Sort();                
                for (int i = 0; i < items.Length; i++)
                {
                    filterListTemp.Add(items[i].ToString());
                }
                for (int i = 0; i < filterItems.Count; i++)
                {
                    if (!filterListItems.Contains(filterItems[i]))
                    {
                        filterListTemp.Add(filterItems[i].ToString());
                    }
                }
                filterListTemp.Sort();
                items = filterListTemp.ToArray();                
            }
        }

        /// <summary>
         /// To get the filtered record 
        /// </summary>
         /// <param name="colName">Filtered Column Name</param>
        /// <param name="sty">GridTableCellStyleInfo</param>
        /// <returns>List of filtered records</returns>
        private List<string> GetFilteredRecord(string colName, GridTableCellStyleInfo sty)
        {
            List<string> choicesOrdered = new List<string>();
            List<string> choices = new List<string>();
            string format = (sty.Format != null && sty.Format.Length > 0)
                    ? string.Format("{{0:{0}}}", sty.Format) : "{0}";

            foreach (Record o in sty.TableCellIdentity.Table.FilteredRecords)
            {
                string val = string.Format(format, o.GetValue(colName));
                int loc = choicesOrdered.BinarySearch(val);
                if (loc < 0)
                {
                    choicesOrdered.Insert(-loc - 1, val);
                    choices.Add(val);
                }
            }
            return choices;
        }

        /// <summary>
        /// To get the all records
        /// </summary>
        /// <param name="colName">Filtered Column Name</param>
        /// <param name="sty">GridTableCellStyleInfo</param>
        /// <returns>List of records</returns>
        private object[] GetRecord(string colName, GridTableCellStyleInfo sty)
        {
            List<string> choicesOrdered = new List<string>();
            List<string> choices = new List<string>();
            string format = (sty.Format != null && sty.Format.Length > 0)
                    ? string.Format("{{0:{0}}}", sty.Format) : "{0}";

            foreach (Record o in sty.TableCellIdentity.Table.Records)
            {
                string val = string.Format(format, o.GetValue(colName));
                int loc = choicesOrdered.BinarySearch(val);
                if (loc < 0)
                {
                    choicesOrdered.Insert(-loc - 1, val);
                    choices.Add(val);
                }
            }
            choices.Sort();
            return choices.ToArray();
        }

        /// <summary>
        /// Used internally
        /// </summary>
        /// <param name="items">An object array of items</param>
        /// <returns>List of string type</returns>
        private List<string> GetFilterChoicesInStringList(object[] items)
        {
            List<string> stringFilterList = new List<string>();
            foreach (object item in items)
            {
                stringFilterList.Add(item.ToString());
            }
            return stringFilterList;
        }

        /// <summary>
        /// Performs sorting and filtering based on the ToolStrip item's selection.
        /// </summary>
        /// <param name="sender">ToolStrip item</param>
        /// <param name="e">event date</param>
        private void ItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.filterDialog.Items.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            switch (index)
            {
                case 0:
                    this.sortDesc.Checked = false;
                    if (this.sortAsc.Checked)
                    {
                        tableDes.SortedColumns.Clear();
                        tableDes.SortedColumns.Add(columnName);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 1:
                    this.sortAsc.Checked = false;
                    if (this.sortDesc.Checked)
                    {
                        tableDes.SortedColumns.Clear();
                        tableDes.SortedColumns.Add(columnName, ListSortDirection.Descending);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 3:
                    if (filterLists.ContainsKey(filterName))
                    {
                        filterLists.Remove(filterName);
                        sty.TableCellIdentity.Table.TableDirty = true;
                        this.Grid.Refresh();
                    }
                    if (filteredColumnCollection.ContainsKey(filterName))
                        filteredColumnCollection.Remove(filterName);
                    sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                    break;
            }

        }

        void filterDialog_Opening(object sender, CancelEventArgs e)
        {
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            if (tableDes.SortedColumns.Contains(columnName))
            {
                if (tableDes.SortedColumns[columnName].SortDirection == ListSortDirection.Ascending)
                {
                    sortAsc.Checked = true;
                    sortDesc.Checked = false;
                }
                else if (tableDes.SortedColumns[columnName].SortDirection == ListSortDirection.Descending)
                {
                    sortAsc.Checked = false;
                    sortDesc.Checked = true;
                }
            }
            else if (sortAsc.Checked || sortDesc.Checked)
            {
                sortAsc.Checked = false;
                sortDesc.Checked = false;
            }
        }

        #endregion

        #region GridExcelFilter control events
        /// <summary>
        /// Hides the filterdialog.
        /// </summary>
        /// <param name="sender">user control</param>
        /// <param name="e">event data</param>
        private void ddUser_UserControlCancel(object sender, EventArgs e)
        {
            this.filterDialog.Hide();
        }
        /// <summary>
        /// Indicates whether the GroupingGrid is hooked with QueryRecordMeetsFilterCriteria event or not
        /// </summary>
        private bool hooked = false;

        /// <summary>
        /// Dictionary for filter choices.
        /// </summary>
        Dictionary<string, List<string>> filterLists = null;

        /// <summary>
        /// Dictionary for filter choices to be exposed.
        /// </summary>
        public Dictionary<string, List<string>> filterCollection = null;

        /// <summary>
        /// Used to get the present filtered column name.
        /// </summary>
        private string filteredColumn = String.Empty;

        private object[] oldItems = null;
		private object[] items = null;
        /// <summary>
        /// Performs filtering and hides the dialog when Ok button clicked.
        /// </summary>
        /// <param name="sender">User control</param>
        /// <param name="e">event data</param>
        private void ddUser_UserControlSave(object sender, EventArgs e)
        {
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;

            string filterName = GetFilterName(sty);
            if (ddUser.GetValues().Count > 0)
            {
                if (!filterLists.ContainsKey(filterName))
                {

                filterLists.Add(filterName, ddUser.GetValues());

                filterLists[filterName].Sort();
                if (!filteredColumnCollection.ContainsKey(filterName))
                        filteredColumnCollection.Add(filterName, this.items);
                }
                else
                {
                    OptimizedFilterDropDownControl opc = sender as OptimizedFilterDropDownControl;
                    filterLists[filterName] = ddUser.GetValues();
                    filterLists[filterName].Sort();
                    if (filteredColumnCollection.ContainsKey(filterName))
                    {
                        filteredColumnCollection.Remove(filterName);
                        if (opc.checkedListBox1.GetItemCheckState(0) == CheckState.Indeterminate)
                            filteredColumnCollection.Add(filterName, this.items);
                    }
                    if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                    if (opc.checkedListBox1.GetItemCheckState(0) == CheckState.Checked)
                        filterLists.Remove(filterName);
                }
            }

            if (!hooked)
            {
                hooked = true;
                GridGroupingControl groupingGrid = ((GridTableModel)Model.Grid).Table.Engine.ParentControl as GridGroupingControl;
                groupingGrid.QueryRecordMeetsFilterCriteria += new QueryRecordMeetsFilterCriteriaEventHandler(ggc_QueryRecordMeetsFilterCriteria);
            }

            Cursor.Current = Cursors.WaitCursor;
            sty.TableCellIdentity.Table.TableDirty = true;
            this.Grid.Refresh();
            Cursor.Current = Cursors.Default;
            filterCollection = filterLists;
            filteredColumn = filterName;
            oldItems = this.items;            
            this.filterDialog.Hide();
        }
        string filterColumnName;
        /// <summary>
        /// Occurs when a record is checked whether it meets filter criteria and should appear visible in the tables DisplayElements.
        /// </summary>
        /// <param name="sender">grid</param>
        /// <param name="e">event data</param>
        private void ggc_QueryRecordMeetsFilterCriteria(object sender, QueryRecordMeetsFilterCriteriaEventArgs e)
        {
            e.Handled = true;
            e.Result = true;
            string name = e.Record.ParentChildTable.Name;
            GridTableDescriptor tableDescriptor = e.Record.ParentChildTable.ParentTableDescriptor as GridTableDescriptor;

            foreach (string filterName in filterLists.Keys)
            {
                if (tableDescriptor.Columns.Contains(filterName))
                {
                    string cellFormat = tableDescriptor.Columns[filterName].Appearance.AnyRecordFieldCell.Format;
                    string format = cellFormat.Length == 0 ? "{0}" : string.Format("{{0:{0}}}", cellFormat);
                    filterColumnName = filterName;

                    List<string> thelist = filterLists[filterName];
                    if (thelist == null || thelist.Count == 0)
                    {
                        continue;
                    }
                    string val = string.Empty;
                    DateTime recDate;
                    if (tableDescriptor.Columns[filterName].Appearance.AnyRecordFieldCell.CellValueType.Equals(typeof(DateTime)) && DateTime.TryParse(e.Record.GetValue(filterName).ToString(), out recDate))
                        val = string.Format(format, recDate);
                    else
                        val = string.Format(format, e.Record.GetValue(filterName));
                    if (val.Equals(string.Empty))
                        val = "(Blanks)";
                    int loc = thelist.BinarySearch(val);
                    e.Result = loc > -1;
                    if (!e.Result)
                    {
                        break;
                    }
                }
            }
            if (e.Result && tableDescriptor.RecordFilters.Count > 0)
                e.Handled = false; //if grid also has record filters (from custom filters) check those too.
        }

        #endregion

        #region CellOverriden Methods
        /// <summary>
        /// Disposes the groupingGrid and its events.
        /// </summary>
        /// <param name="disposing">the boolean flag</param>
        protected override void Dispose(bool disposing)
        {
            GridGroupingControl groupingGrid = ((GridTableModel)Model.Grid).Table.Engine.ParentControl as GridGroupingControl;
            if (hooked && groupingGrid != null)
            {
                hooked = !hooked;
                groupingGrid.QueryRecordMeetsFilterCriteria -= new QueryRecordMeetsFilterCriteriaEventHandler(ggc_QueryRecordMeetsFilterCriteria);
            }

            if (Grid != null)
                this.Grid.CellClick -= new GridCellClickEventHandler(Grid_CellClick);

            base.Dispose(disposing);
        }
        /// <summary>
        /// Returns the exact image of the bitmap name.
        /// </summary>
        /// <param name="bitmapName">name of the bitmap</param>
        /// <returns>The image</returns>
        private string FindImageFile(string bitmapName)
        {
            string bitmappath = "";
            for (int n = 0; n < 10; n++)
            {
                if (System.IO.File.Exists(bitmapName))
                    bitmappath = bitmapName;

                bitmapName = @"..\" + bitmapName;
            }
            return bitmappath;
        }
        /// <summary>
        /// Occurs when mouse hover enters the column header.
        /// </summary>
        /// <param name="rowIndex">row index</param>
        /// <param name="colIndex">col index</param>
        protected override void OnMouseHoverEnter(int rowIndex, int colIndex)
        {
            IsMouseOver = true;
            colindex = colIndex;
            this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
            base.OnMouseHoverEnter(rowIndex, colIndex);
        }
        /// <summary>
        /// Occurs when mouse hover leaves the column header.
        /// </summary>
        /// <param name="rowIndex">row index</param>
        /// <param name="colIndex">col index</param>
        /// <param name="e"></param>
        protected override void OnMouseHoverLeave(int rowIndex, int colIndex, EventArgs e)
        {
            IsMouseOver = false;
            colindex = colIndex;
            this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
            base.OnMouseHoverLeave(rowIndex, colIndex, e);
        }

        
        protected override void OnDrawDisplayText(Graphics g, Rectangle textRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            object tag = style.Tag;
            if (Grid.PrintingMode || !(tag is ListSortDirection))
            {
                tag = null;
            }

            ListSortDirection listSortDirection = ListSortDirection.Ascending;
            int margin = filterWidth;
            if (tag != null)
            {
                listSortDirection = (ListSortDirection)tag;
                margin = filterWidth + 12;
            }

            bool isTextRightToLeft = Grid.SortIconPlacement == SortIconPlacement.Left;
            bool isTextTop = Grid.SortIconPlacement == SortIconPlacement.Top;
            if (isTextRightToLeft || Grid.IsRightToLeft() || style.RightToLeft == RightToLeft.Yes)
            {
                GridUtil.OffsetLeft(ref textRectangle, margin);
            }
            else if (isTextTop)
            {
                GridUtil.OffsetTop(ref textRectangle, 0);
            }
            else
            {
                textRectangle.Width -= margin;
            }

            base.OnDrawDisplayText(g, textRectangle, rowIndex, colIndex, style);
           
            #region ImageRendering
            Rectangle imageRect;

            if (Grid.IsRightToLeft() || isTextRightToLeft || style.RightToLeft == RightToLeft.Yes)
            {
                imageRect = new Rectangle(textRectangle.Left - filterWidth, textRectangle.Y + (textRectangle.Height / 6), filterWidth, filterHeight);
            }
            else
            {
                imageRect = new Rectangle(textRectangle.Right, textRectangle.Y + (textRectangle.Height / 6), filterWidth, filterHeight);
            }
            GridTableCellStyleInfo tablestyle = style as GridTableCellStyleInfo;
            Image imag_gray = (Image)filterImages["Filter_gray"];
            Image clImage_gray = (Image)filterImages["ClearFilter_gray"];
            Image imag_white = (Image)filterImages["Filter_white"];
            Image clImage_white = (Image)filterImages["ClearFilter_white"];

            if (tablestyle.TableCellIdentity.Column != null && tablestyle.TableCellIdentity.Column.AllowFilter == true)
            {
                GridTableDescriptor tableDesc = ((GridTableControl)this.Grid).TableDescriptor;
                bool isFiltered = false;
                if (filterLists.ContainsKey(tablestyle.TableCellIdentity.Column.Name)
                    /*&& GridOffice2007Filter.EnableFilteredColumnIcon*/)
                {
                    isFiltered = true;
                }

                if (GridExcelFilter.EnableFilteredColumnIcon)
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = imageRect;
                    }
                    if(IsMouseOver && colindex == colIndex && !isFiltered)
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(imag_white, imageRect);
                        else
                            g.DrawImage(imag_gray, imageRect);
                    }
                    
                }
                else
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = imageRect;
                    }

                    if (!isFiltered)
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(imag_white, imageRect);
                        else
                            g.DrawImage(imag_gray, imageRect);
                    }
                }
                if (GridExcelFilter.EnableFilteredColumnIcon || isFiltered)
                {
                    foreach (string filterName in filterLists.Keys)
                    {
                        if (IsMouseOver && colindex == colIndex)
                        {
                            filterRect = imageRect;
                        }
                        if (filterName == tablestyle.TableCellIdentity.Column.Name)
                        {
                            if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                                g.DrawImage(clImage_white, imageRect);
                            else
                                g.DrawImage(clImage_gray, imageRect);
                            isFiltered = false;
                        }
                    }
                }
                
            }

            #endregion

            if (tag != null)
            {
                string s = style.ValueMember; 
                int dig = (!string.IsNullOrEmpty(s) && s.Length > 1) ? 2 : 1;

                Rectangle rect;
                if (isTextRightToLeft)
                {
                    rect = new Rectangle(textRectangle.Left - margin, textRectangle.Y, 10, textRectangle.Height);
                }
                else if (isTextTop)
                {
                    rect = new Rectangle(textRectangle.X, textRectangle.Y - 8, textRectangle.Width, textRectangle.Height);
                }
                else
                {
                    rect = new Rectangle(textRectangle.Right + filterWidth, textRectangle.Y, 10, textRectangle.Height);
                }

                rect = GridUtil.CenterInRect(rect, new Size(8 * dig, 8));

                Brush brush = null;
                Pen pen1 = null;
                this.Grid.Model.Options.GridVisualStylesDrawing.GetSortIconBrush(out brush, out pen1);
                if (s != string.Empty)
                {
                    rect.Offset(-4 * (dig - 1), -4);
                    Font numFont = new Font(style.Font.Facename, 6, style.Font.FontStyle);
                    if (isTextTop)
                        DrawText(g, s, numFont, new Rectangle(rect.X + 5, rect.Y + 5, rect.Width, rect.Height), style, pen1.Color, Grid.IsRightToLeft());
                    else
                        DrawText(g, s, numFont, rect, style, pen1.Color, Grid.IsRightToLeft());
                    numFont.Dispose();
                    rect.Offset(4 * (dig - 1), 4);

                    if (isTextRightToLeft || Grid.IsRightToLeft())
                    {
                        rect.Offset(6, 0);
                    }
                    else
                    {
                        rect.Offset(-6, 0);
                    }
                }

                int i2 = Math.Max(0, (rect.Height - 6) / 2);
                rect.Inflate(-i2, -i2);
                GridTriangleDirection triangleDirection = listSortDirection == ListSortDirection.Ascending ? GridTriangleDirection.Up : GridTriangleDirection.Down;
                GridPaintTriangle.Paint(g, rect, triangleDirection, brush, pen1, true);
                pen1.Dispose();
                brush.Dispose();
            }
        }

        protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
            Color clrBottom = Color.Empty;
            Color clrRight = Color.Empty;
            Color clrInteriorFirst = Color.Empty;
            Color clrInteriorLast = Color.Empty;

            if (this.Grid.Model.Options.GridVisualStylesDrawing.GetHeaderBorderColors(out clrBottom, out clrRight, out clrInteriorFirst, out clrInteriorLast) &&
               ((style.Themed && XPThemes.IsThemedOS && XPThemes.IsThemeActive && this.Grid.ThemesEnabled)
                      || ((style.Themed && this.Grid.ThemesEnabled && ((this.Grid.Model.Options.GridVisualStyles != GridVisualStyles.SystemTheme))))))
            {
                Rectangle rect1 = clientRectangle;
                Pen br = new Pen(clrBottom);
                g.DrawLine(br, new PointF(rect1.X, rect1.Y), new PointF((rect1.X + rect1.Width), rect1.Y));
                br.Dispose();
            }
        }

        /// <summary>
        ///  Occurs when the user clicks inside the  cell boundary.
        /// </summary>
        /// <param name="sender">grid</param>
        /// <param name="e">event data</param>
        private void Grid_CellClick(object sender, GridCellClickEventArgs e)
        {
            if (filterRect.IntersectsWith(new Rectangle(e.MouseEventArgs.X, e.MouseEventArgs.Y, 0, 0)))
            {
                GridRangeInfo range = GridRangeInfo.Cell(e.RowIndex, e.ColIndex);
                Rectangle rec = this.Grid.RangeInfoToRectangle(range);
                Point loc = new Point(rec.Right, rec.Bottom);
                Point locn = new Point(e.MouseEventArgs.X, e.MouseEventArgs.Y);
                GridTableControl grid = this.Grid as GridTableControl;
                filterRow = e.RowIndex;
                Filtercol = e.ColIndex;
                GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;

                clearFilter.Text = clearFilterString + " " + sty.TableCellIdentity.Column.HeaderText;

                string filterName = GetFilterName(sty);
                if (filteredColumnCollection.ContainsKey(filterName))
                {
                    filteredColumnCollection.TryGetValue(filterName, out items);
                    if (isCurrentCellEdited)
                    {
                        GenarateFilterColumnCollections(filterName, sty);
                        isCurrentCellEdited = false;
                    }
                }
                else
                    items = GetFilterBarChoices(sty.TableCellIdentity);
                if ((filterLists != null && filterLists.ContainsKey(filterName)) || sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))                                   
                    clearFilter.Enabled = true;
                else
                    clearFilter.Enabled = false;
                IList filteredValues = this.GetFilterTextList(sty);
                GridTableCellStyleInfo formatStyle = new GridTableCellStyleInfo();
                formatStyle = sty;
                formatStyle.CellValueType = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.CellValueType;
                formatStyle.CellValue = sty.CellValue;
                formatStyle.Format = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.Format;
                if (sty.GetActiveGridView() != null && sty.GetActiveGridView().GridOfficeScrollBars == Syncfusion.Windows.Forms.OfficeScrollBars.Metro)
                {
                    this.filterDialog.Font = new Font("Segoe UI", 8.25f);
                    this.filterDialog.ForeColor = Color.FromArgb(51, 51, 51);
                    this.ddUser.checkedListBox1.BorderStyle = BorderStyle.FixedSingle;
                }
                WireButton(this.ddUser.okButton);
                WireButton(this.ddUser.cancelButton);
                this.ddUser.SetItems(items, filteredValues, formatStyle);
                this.filterDialog.Show(grid, locn);
                e.Cancel = true;
            }
        }
        /// <summary>
        /// Sets Metro button.
        /// </summary>
        /// <param name="button">button control</param>
        private void WireButton(ButtonAdv button)
        {
           
            if (button.Enabled)
                button.BackColor = Color.FromArgb(253, 143, 0);
            button.Font = new Font("Segoe UI", 8.25f, FontStyle.Bold);

            switch (this.Grid.GetGridVisualStylesDrawing().VisualStyle.ToString())
            {
                case "SystemTheme":
                    button.Appearance = Windows.Forms.ButtonAppearance.None;                    
                    break;
                case "Metro":
                    button.Appearance = Windows.Forms.ButtonAppearance.Metro;
                    button.BackColor = Color.FromArgb(253, 143, 0);
                    button.ForeColor = Color.White;                   
                    break;
                case "Custom":
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Managed;
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Managed;
                    break;
                case "Office2003":
                    button.Appearance = Windows.Forms.ButtonAppearance.Office2003;
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Blue;
                    break;
                case "Office2007Blue":
                case "Office2010Blue":
                    button.Appearance = Windows.Forms.ButtonAppearance.Office2007;                    
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Blue;                    
                    break;
                case "Office2007Black":
                case "Office2010Black":
                    button.Appearance = Windows.Forms.ButtonAppearance.Office2007;                    
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Black;                    
                    break;
                case "Office2007Silver":
                case "Office2010Silver":
                    button.Appearance = Windows.Forms.ButtonAppearance.Office2007;                    
                    button.Office2007ColorScheme = Windows.Forms.Office2007Theme.Silver;                   
                    break;
            }
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Returns an array of category keys for this group and all parent groups which is used by FilterBarCells and FilterBarSummary
        /// to compare whether the conditions should be applied to this group.
        /// </summary>
        /// <param name="tableCellIdentity">Cell identifier.</param>
        /// <returns>An array of category keys.</returns>
        public object[] GetUniqueGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            return g.UniqueGroupId;
        }

        /// <summary>
        /// Returns the unique choices to be displayed in the filterbar cell.
        /// </summary>
        /// <param name="tableCellIdentity">Table cell identifier.</param>
        /// <returns>Filter bar choices.</returns>
        public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            GridGroupingControl groupGrid = ((GridEngine)tableCellIdentity.Table.Engine).GroupingControl;
            object[] result = null;
            GridGroupingControl gc = tableCellIdentity.Table.Engine.ParentControl;
            GridQueryFilterBarChoicesEventArgs qe = new GridQueryFilterBarChoicesEventArgs(tableCellIdentity.Column, false, tableCellIdentity.DisplayElement);
            gc.OnQueryFilterBarChoices(qe);
            if (!qe.Cancel)
            {
                result = qe.UniqueFilterBarValues;
            }
            Array.Sort(result, new ValueComparer());
            return result;
        }

        /// <summary>
        /// Determines the name of the filter based on the cell style information passed in.
        /// </summary>
        /// <param name="sty">cell style information.</param>
        /// <returns>Name of the filter.</returns>
        public string GetFilterName(GridTableCellStyleInfo sty)
        {
            object[] id = GetUniqueGroupId(sty.TableCellIdentity);
            string filterName = "";
            if (id == null || id.Length == 0)
            {
                filterName = sty.TableCellIdentity.Column.Name;
            }
            else
            {
                filterName = sty.TableCellIdentity.Column.Name + "@" + sty.TableCellIdentity.DisplayElement.ParentGroup.UniqueGroupIdsToString();
            }
            return filterName;
        }

        /// <summary>
        /// Gets the filter values collection.
        /// </summary>
        /// <param name="sty">cell style</param>
        /// <returns>filter values</returns>
        private IList GetFilterTextList(GridTableCellStyleInfo sty)
        {
            List<string> filterValues = new List<string>();
            filterValues.Clear();
            string filterName = GetFilterName(sty);
            if (!GridUtil.IsEmpty(filterName) && filterLists.ContainsKey(filterName))
            {
                filterValues = filterLists[filterName];
            }
            return filterValues;
        }

        /// <summary>
        /// Clears all the filter choices from dictionary.
        /// </summary>
        public void ClearFilters()
        {
            filterLists.Clear();
        }
        #endregion
    }
}
