﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Syncfusion.Windows.Forms.Tools;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms.Grid;

namespace Syncfusion.GridHelperClasses
{
    /// <summary>
    /// User control to provide the design for optimized excel like filter dialog.
    /// </summary>
    internal partial class OptimizedFilterDropDownControl : UserControl
    {
        public event EventHandler UserControlSave;
        public event EventHandler UserControlCancel;
        private bool initDone = false;
        /// <summary>
        /// used to indicate changes to the values
        /// </summary>
        private bool changed = false;
       /// <summary>
       /// Constructor for excel like filter dialog.
       /// </summary>
        public OptimizedFilterDropDownControl(string style)
        {
            InitializeComponent();            
           
            checkedListBox1.ItemCheck += new ItemCheckEventHandler(checkedListBox1_ItemCheck);
            checkedListBox1.CheckOnClick = true;
             
            checkedListBox1.Items.Add(SR.GetString(SR.SelectAll));
        }

        /// <summary>
        /// Indicates the check box checking.
        /// </summary>
        private bool inItemCheck = false;
        /// <summary>
        /// Occurs when checked listbox clicked.
        /// </summary>
        /// <param name="sender">check box</param>
        /// <param name="e">event data</param>
        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (!initDone || inItemCheck)
                return;
            changed = true;
            inItemCheck = true;
            if (e.Index == 0)
            {
                this.okButton.Enabled = e.NewValue == CheckState.Unchecked ? false : true;
                for (int i = 1; i < checkedListBox1.Items.Count; i++)
                {
                    checkedListBox1.SetItemCheckState(i, e.NewValue);
                }
            }
            else
            {
                if (e.NewValue == CheckState.Checked)
                {
                    this.checkedListBox1.SetItemCheckState(0, CheckState.Indeterminate);
                }
                else if (e.NewValue == CheckState.Indeterminate)
                {
                    this.checkedListBox1.SetItemCheckState(e.Index, CheckState.Unchecked);
                }
             }
            //delay ResetSelectAll so new check action is completed...
           // this.checkedListBox1.BeginInvoke(new System.Action(() => {ResetSelectAll(e);}), null); //requires .Net 3.5
            this.checkedListBox1.BeginInvoke(new ResetSelectAllDelegate(ResetSelectAll), new object[] { e }); //.NET 2.0 code
           
            inItemCheck = false;
        }

        /// <summary>
        /// Delegate for ResetSelectAll method.
        /// </summary>
        /// <param name="e">event data</param>
        private delegate void ResetSelectAllDelegate(ItemCheckEventArgs e);
        /// <summary>
        /// Resets the filter choices.
        /// </summary>
        /// <param name="e">event data</param>
        private void ResetSelectAll(ItemCheckEventArgs e)
        {
            inItemCheck = true;
            if (e.Index == 0)
            {
                if (e.NewValue == CheckState.Indeterminate)
                    checkedListBox1.SetItemCheckState(0, CheckState.Unchecked);
                if(checkedListBox1.GetItemCheckState(0) == CheckState.Unchecked)
                    this.checkedListBox1.ClearSelected();
            } 
            else if (checkedListBox1.CheckedIndices.Count == 1 && e.NewValue == CheckState.Unchecked && checkedListBox1.GetItemCheckState(0) != CheckState.Unchecked)
            {
                checkedListBox1.SetItemCheckState(0, CheckState.Unchecked);
            }
            else if (checkedListBox1.CheckedIndices.Count == checkedListBox1.Items.Count && e.Index != 0 && e.NewValue == CheckState.Checked && checkedListBox1.GetItemCheckState(0) != CheckState.Checked)
            {
                checkedListBox1.SetItemCheckState(0, CheckState.Checked);
            }
            else if (checkedListBox1.CheckedIndices.Count < checkedListBox1.Items.Count)
            {
                checkedListBox1.SetItemCheckState(0, CheckState.Indeterminate);
            }

            this.okButton.Enabled = this.checkedListBox1.GetItemCheckState(0) == CheckState.Unchecked ? false : true;
            inItemCheck = false;
        }

        /// <summary>
        /// Occurs when check state of the CheckBox has changed.
        /// </summary>
        /// <param name="sender">check box</param>
        /// <param name="e">event data</param>
        private void DropDownFilter_CheckStateChanged(object sender, EventArgs e)
        {
            if (!this.initDone)
                return;
            changed = true;            
            this.okButton.Enabled = this.checkedListBox1.GetItemCheckState(0) == CheckState.Unchecked  ? false : true;
        }

        ArrayList itemCollection = new ArrayList();
        /// <summary>
        /// Adds the filter choices to the checked listbox.
        /// </summary>
        /// <param name="items">filter choices</param>
        /// <param name="filteredValues">filtered values</param>
        /// <param name="formatStyle">cell style</param>
        public void SetItems(object[] items, IList filteredValues, GridTableCellStyleInfo formatStyle)
        {

            this.initDone = false;
            checkedListBox1.Items.Clear();
            checkedListBox1.Items.Add(SR.GetString(SR.SelectAll));
            bool allChecked = true, intermidiate = false;
            List<string> values = new List<string>();
            foreach (object it in items)
            {
                string formattedValue = string.Empty;
                if (formatStyle.CellValueType == typeof(System.DateTime) && it != System.DBNull.Value)
                {
                    formattedValue = formatStyle.GetFormattedText(Convert.ToDateTime(it));
                }
                else
                {
                    formattedValue = formatStyle.GetFormattedText(it);
                }
                if (!values.Contains(formattedValue))
                {
                    if (formattedValue.Equals(string.Empty))
                        formattedValue = "(Blanks)";
                    int loc = checkedListBox1.Items.Add(formattedValue);
                    if (filteredValues.Contains(formattedValue) || filteredValues.Count == 0)
                    {
                        if (formatStyle.TableCellIdentity != null && formatStyle.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(formatStyle.Text))
                        {
                            checkedListBox1.SetItemCheckState(loc, CheckState.Unchecked);
                            checkedListBox1.SetItemCheckState(0, CheckState.Unchecked);
                        }
                        else
                        {
                            checkedListBox1.SetItemCheckState(loc, CheckState.Checked);
                            intermidiate = true;
                        }
                    }
                    else
                        allChecked = false;
                    values.Add(formattedValue);
                }
            }

            if (checkedListBox1.Items.Contains("(Blanks)"))
            {
                int chkIndex = checkedListBox1.Items.IndexOf("(Blanks)");
                bool chkState = checkedListBox1.GetItemChecked(chkIndex);
                this.checkedListBox1.Items.Remove("(Blanks)");
                this.checkedListBox1.Items.Insert(this.checkedListBox1.Items.Count, "(Blanks)");
                int chkedIndex = checkedListBox1.Items.IndexOf("(Blanks)");
                if (chkState)
                    checkedListBox1.SetItemCheckState(chkedIndex, CheckState.Checked);
            }

            if (allChecked && intermidiate)
            {
                checkedListBox1.SetItemCheckState(0, CheckState.Checked);
            }
            if (!allChecked && intermidiate)
            {
                checkedListBox1.SetItemCheckState(0, CheckState.Indeterminate);
            }
            this.initDone = true;
            this.okButton.Enabled = GetValues().Count > 0 ? true : false;
        }

        /// <summary>
        /// Gets the choices from the checked listbox
        /// </summary>
        /// <returns>the checked items list</returns>
        public List<string> GetValues()
        {
            List<string> checkedItem = new List<string>();

            //start at one to skip SelectAll
             for(int i = 1; i < checkedListBox1.Items.Count; ++i)
            {
                if(checkedListBox1.GetItemCheckState(i) == CheckState.Checked)
                    checkedItem.Add(checkedListBox1.Items[i].ToString());
            }
            return checkedItem;
        }

        /// <summary>
        /// Performs filter operation for Ok button.
        /// </summary>
        /// <param name="sender">Ok button</param>
        /// <param name="e">event data</param>
        private void button1_Click(object sender, EventArgs e)
        {
            bool saveChanged = this.changed;
            this.changed = false;
            if (saveChanged && UserControlSave != null)
                UserControlSave(this, EventArgs.Empty);
            else if (UserControlCancel != null)
                UserControlCancel(this, EventArgs.Empty);
        }

        /// <summary>
        /// Performs cancel operation.
        /// </summary>
        /// <param name="sender">Cancel button</param>
        /// <param name="e">event data</param>
        private void button2_Click(object sender, EventArgs e)
        {
            changed = false;
            if (UserControlCancel != null)
                UserControlCancel(this, EventArgs.Empty);
        }
    }
}
