﻿using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.Windows.Forms.Grid.Grouping;
using System.ComponentModel;
using Syncfusion.Grouping;

namespace Syncfusion.GridHelperClasses
{  
    /// <summary>
    /// Implements Office2007 filter.
    /// </summary>
    public class GridOffice2007Filter
    {
        /// <summary>
        /// Intializes a new <see cref="GridOffice2007Filter"/>
        /// </summary>
        public GridOffice2007Filter()
            : base()
        {
        }

        # region ShowOffice2007FilterOnMouseHover
        private static bool showOffice2007FilterOnMouseHover = true;
        /// <summary>
        /// set this property to enable the Office2007 filter on mouse over.if false is set, the filter displays directly when the grid has been wired.
        /// </summary>
        [DefaultValue("true")]
        public static bool ShowOffice2007FilterOnMouseHover
        {
            get
            {
                return showOffice2007FilterOnMouseHover;
            }
            set
            {
                if (showOffice2007FilterOnMouseHover != value)
                {
                    showOffice2007FilterOnMouseHover = value;
                }
            }
        }
        # endregion

        #region EnableFilteredColumnIcon
        private static bool enableFilteredColumnIcon = false;
        /// <summary>
        /// Gets or Sets this property to stick the filter icon in the filtered column. 
        /// This property is effective only if the ShowOffice2007FilterOnMouseHover property is in true.
        /// </summary>
        [DefaultValue(false)]
        public static bool EnableFilteredColumnIcon
        {
            get
            {
                return enableFilteredColumnIcon;
            }
            set
            {
                if (enableFilteredColumnIcon != value)
                {
                    enableFilteredColumnIcon = value;
                }
            }
        }
        #endregion

        /// <summary>
        /// Hook the grouping grid to the office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void WireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {                
                groupingGrid.BeginUpdate();
                groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new Grid2007ExcelFilterCellModel(groupingGrid.TableModel);            
                groupingGrid.TableDescriptor.IsExcelFilterWired = true;
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
            }
        }

        /// <summary>
        /// Unhook the grouping grid from the office2007 filter.
        /// </summary>
        /// <param name="groupingGrid">The grouping grid.</param>
        public void UnWireGrid(GridGroupingControl groupingGrid)
        {
            if (groupingGrid != null)
            {
                groupingGrid.BeginUpdate();
                groupingGrid.TableModel.CellModels["ColumnHeaderCell"] = new GridTableColumnHeaderCellModel(groupingGrid.TableModel);
                groupingGrid.TableDescriptor.IsExcelFilterWired = false;
                groupingGrid.EndUpdate(true);
                groupingGrid.Refresh();
                groupingGrid = null;
            }
        }
    }
}
