#region Copyright Syncfusion Inc. 2001 - 2009
// Copyright Syncfusion Inc. 2001 - 2009. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Data;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using Syncfusion.Grouping; 
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Windows.Forms;
using Syncfusion.Collections.BinaryTree;
using System.Collections.Generic;

namespace Syncfusion.GridHelperClasses
{
    /// <summary>
    /// Implements the DataModel part for a ExcelFilterCell.
    /// </summary>
	public class Grid2007ExcelFilterCellModel : GridHeaderCellModel
	{
        /// <summary>
        /// Initializes a new <see cref="Grid2007ExcelFilterCellModel"/> object
        /// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
        /// </summary>
        /// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>
        /// <remarks>
        /// You typically access cell models through the <see cref="GridModel.CellModels"/>
        /// property of the <see cref="GridModel"/> class.
        /// </remarks>
		public Grid2007ExcelFilterCellModel(GridModel grid)
			: base(grid)
		{
		}

		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new Grid2007ExcelFilterCellRenderer(control, this);
		}
	}
    /// <summary>
    /// Implements the renderer part of a ExcelFilterCell.
    /// </summary>
	public class Grid2007ExcelFilterCellRenderer: GridHeaderCellRenderer
	{
        private FilterDropDownControl ddUser;
		Rectangle filterRect = Rectangle.Empty;   
		bool IsMouseOver = false;
		int colindex = -1;		                	
        private List<string> resizedColumns = new List<string>();
        Hashtable filterImages;
        int filterRow=-1, Filtercol=-1;
        ToolStripMenuItem sortAsc;
        ToolStripMenuItem sortDesc;
        ToolStripMenuItem clearFilter;
        ToolStripMenuItem textFilters;
        ToolStripMenuItem equal;
        ToolStripMenuItem notEqual;
        ToolStripMenuItem beginsWith;
        ToolStripMenuItem endsWith;
        ToolStripMenuItem contains;        
        ToolStripMenuItem customFilter;
        
        ContextMenuStrip filterDialog;        
        string clearFilterString = SR.GetString(SR.ClearFilterFrom);
        int filterHeight = 16, filterWidth = 16;

        #region Constructor

        /// <summary>
        /// Initializes a new <see cref="Grid2007ExcelFilterCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridCellModelBase"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="Grid2007ExcelFilterCellModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>
        /// References to GridControlBase and GridCellModelBase will be saved.
        /// </remarks>
        public Grid2007ExcelFilterCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
            filterImages = new Hashtable();
            filterImages.Add("Filter_gray", DynamicFilterBitmaps.GetBitmap("filter_funnel_gr"));
            filterImages.Add("ClearFilter_gray", DynamicFilterBitmaps.GetBitmap("filter_set_gr"));
            filterImages.Add("Filter_white", DynamicFilterBitmaps.GetBitmap("filter_metro"));
            filterImages.Add("ClearFilter_white", DynamicFilterBitmaps.GetBitmap("filter_set_wh"));
            //filterImages.Add("RemoveFilter", );

            filterDialog = new ContextMenuStrip();

            this.ddUser = new FilterDropDownControl();
            //hook the usercontrol save and cancel events...
            this.ddUser.UserControlSave += new EventHandler(ddUser_UserControlSave);
            this.ddUser.UserControlCancel += new EventHandler(ddUser_UserControlCancel);
            this.ddUser.Dock = DockStyle.Fill;
            sortAsc = new ToolStripMenuItem(SR.GetString(SR.SortAtoZ), DynamicFilterBitmaps.GetBitmap("sortasc"), ItemSelection_Click);
            sortAsc.CheckOnClick = true;            
            this.filterDialog.Items.Add(sortAsc);
            sortDesc = new ToolStripMenuItem(SR.GetString(SR.SortZtoA), DynamicFilterBitmaps.GetBitmap("sortdesc"), ItemSelection_Click);
            sortDesc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortDesc);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            this.clearFilter = new ToolStripMenuItem(SR.GetString(SR.ClearFilterFrom), DynamicFilterBitmaps.GetBitmap("filter_delete"), ItemSelection_Click);            
            this.filterDialog.Items.Add(clearFilter);
            textFilters = new ToolStripMenuItem(SR.GetString(SR.TextFilters));
            

            equal = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEquals), null, CustomItemSelection_Click);
            notEqual = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterNotEquals), null, CustomItemSelection_Click);
            beginsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterBeginswith), null, CustomItemSelection_Click);
            endsWith = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEndswith), null, CustomItemSelection_Click);
            contains = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterContains), null, CustomItemSelection_Click);
            customFilter = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterCustomFilter), null, CustomItemSelection_Click);
            textFilters.DropDownItems.Add(equal);
            textFilters.DropDownItems.Add(notEqual);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(beginsWith);
            textFilters.DropDownItems.Add(endsWith);            
            textFilters.DropDownItems.Add(contains);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(customFilter);
            this.filterDialog.Items.Add(textFilters);
            this.filterDialog.Items.Add(new ToolStripSeparator());                        
            ToolStripControlHost filterDropDownStrip = new ToolStripControlHost(this.ddUser);            
            this.filterDialog.Items.Add(filterDropDownStrip);
            this.ddUser.BackColor = filterDropDownStrip.BackColor;
			this.Grid.CellClick +=new GridCellClickEventHandler(Grid_CellClick);          
            SetColumnWidths(false);
        }

        void CustomItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.textFilters.DropDownItems.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            if (index!=-1)
            {   
                if (this.Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                {
                    MetroCustomRowFilter textFilter = new MetroCustomRowFilter();
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }
                else
                {
                    CustomRowFilter textFilter = new CustomRowFilter();
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }            
            }
        }

        void ItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.filterDialog.Items.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes=sty.TableCellIdentity.Table.TableDescriptor;
            string columnName=sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);                    
            switch (index)
            {
                case 0:
                    this.sortDesc.Checked = false;
                    if (this.sortAsc.Checked)
                    {
                        tableDes.SortedColumns.Remove(columnName);
                        tableDes.SortedColumns.Add(columnName);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 1:
                    this.sortAsc.Checked = false;
                    if (this.sortDesc.Checked)
                    {
                        tableDes.SortedColumns.Remove(columnName);
                        tableDes.SortedColumns.Add(columnName,ListSortDirection.Descending);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 3:                    
                    sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                    break;                
            }

        }
        #endregion

        #region Office2007Filter control events
        void ddUser_UserControlCancel(object sender, EventArgs e)
        {
            this.filterDialog.Hide();
        }

        void ddUser_UserControlSave(object sender, EventArgs e)
        {            
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow,Filtercol] as GridTableCellStyleInfo;        
            IList list = ddUser.GetValues();
            string filterName = GetFilterName(sty);            
            RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName);
            if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
            if (list.Count>0)
            {
                rfc.LogicalOperator =FilterLogicalOperator.Or;
                foreach (object s in list)
                {
                    FilterCondition con = new FilterCondition(FilterCompareOperator.Equals, s);
                     if (s.ToString() == "(Blanks)")
                    {
                        con.CompareText = string.Empty;
                        con.CompareValue = string.Empty;
                        rfc.Conditions.Add(con);
                    }
                    else	
                        rfc.Conditions.Add(con);
                }
                sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
            }            
            this.filterDialog.Hide();
        }

        private void SetColumnWidths(bool dispose)
        {
            this.Grid.BeginUpdate();
            GridTableControl groupingGrid = (GridTableControl)Grid;
            if (groupingGrid != null)
            {
                if (!dispose)
                {
                    GridTableDescriptor tableDesc = groupingGrid.TableDescriptor;
                    foreach (GridColumnDescriptor column in tableDesc.Columns)
                    {
                        if (!resizedColumns.Contains(column.Name))
                        {
                            this.Grid.Model.ColWidths[column.Name] += 15;
                            resizedColumns.Add(column.Name);
                        }

                    }
                }
                else
                {
                    GridTableDescriptor tableDesc = groupingGrid.TableDescriptor;
                    foreach (GridColumnDescriptor column in tableDesc.Columns)
                    {
                        if (resizedColumns.Contains(column.Name))
                        {
                            this.Grid.Model.ColWidths[column.Name] -= 15;
                            resizedColumns.Remove(column.Name);
                        }
                    }
                }
            }
            this.Grid.EndUpdate();
        }
        #endregion

        #region CellOverriden Methods

        private string FindImageFile(string bitmapName)
        {
            string bitmappath = "";
            for (int n = 0; n < 10; n++)
            {
                if (System.IO.File.Exists(bitmapName))
                    bitmappath = bitmapName;

                bitmapName = @"..\" + bitmapName;
            }
            return bitmappath;
        }

        protected override void Dispose(bool disposing)
        {
            if (Grid != null)
            {
                SetColumnWidths(true);
            }
            base.Dispose(disposing);
        }
		protected override void OnMouseHoverEnter(int rowIndex, int colIndex)
		{
			IsMouseOver = true;
			colindex =colIndex; 
			this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
			base.OnMouseHoverEnter (rowIndex, colIndex);
		}
		
		protected override void OnMouseHoverLeave(int rowIndex, int colIndex, EventArgs e)
		{
			IsMouseOver =  false;
			colindex =colIndex;
			this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
			base.OnMouseHoverLeave (rowIndex, colIndex, e);
		}

        protected override void OnDrawDisplayText(Graphics g, Rectangle textRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            textRectangle.Width -= filterWidth + 2;
            base.OnDrawDisplayText(g, textRectangle, rowIndex, colIndex, style);
        }

		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
            Rectangle rect;
            if (Grid.IsRightToLeft())
            {
                rect = new Rectangle(clientRectangle.Left, clientRectangle.Y + (clientRectangle.Height / 6), filterWidth, filterHeight);//clientRectangle.X + clientRectangle.Width - 12, clientRectangle.Y + clientRectangle.Height / 3, 9 , 9 );   
            }
            else
            {
                rect = new Rectangle(clientRectangle.Right - 2 * (filterWidth) + 4, clientRectangle.Y + (clientRectangle.Height / 6) , filterWidth, filterHeight);
            }
			GridTableCellStyleInfo tablestyle = style as GridTableCellStyleInfo;
            Image imag_gray=(Image)filterImages["Filter_gray"];
            Image clImage_gray = (Image)filterImages["ClearFilter_gray"];
            Image imag_white = (Image)filterImages["Filter_white"];
            Image clImage_white = (Image)filterImages["ClearFilter_white"];
            if (tablestyle.TableCellIdentity.Column != null && tablestyle.TableCellIdentity.Column.AllowFilter == true)
            {
                GridTableDescriptor tableDesc = ((GridTableControl)this.Grid).TableDescriptor;
                bool isFiltered = false;
                if (tableDesc != null && tableDesc.RecordFilters.Count > 0 
                    && tableDesc.RecordFilters.Contains(tablestyle.TableCellIdentity.Column.Name)
                    && GridOffice2007Filter.EnableFilteredColumnIcon)
                {
                    isFiltered = true;
                }
                if (GridOffice2007Filter.ShowOffice2007FilterOnMouseHover)
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = rect;
                    }
                     if (isFiltered && tableDesc.RecordFilters.Contains(tablestyle.TableCellIdentity.Column.Name))
                     {
                         if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                             g.DrawImage(clImage_white, rect); 
                         else
                             g.DrawImage(clImage_gray, rect);
                        isFiltered = false;
                    }
                    else if (((IsMouseOver && colindex == colIndex) || (this.Grid.Model.SelectedRanges.AnyRangeIntersects(GridRangeInfo.Cell(rowIndex, colIndex)))))
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(imag_white, rect); 
                        else
                            g.DrawImage(imag_gray, rect);
                    }
                    
                }
                else
                {
                    if (IsMouseOver && colindex == colIndex)
                    {
                        filterRect = rect;
                    }

                    if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                        g.DrawImage(imag_white, rect);
                    else
                        g.DrawImage(imag_gray, rect);
                }
            }
            ShowSortIcon(g, clientRectangle, rowIndex, colIndex, style);
        }
        private void ShowSortIcon(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            // No arrow needed when printing.
            object tag = style.Tag;
            if (Grid.PrintingMode || !(tag is ListSortDirection))
            {
                tag = null;
            }

            ListSortDirection listSortDirection = ListSortDirection.Ascending;
            int margin = 0;
            if (tag != null)
            {
                listSortDirection = (ListSortDirection)tag;
                margin = 12;
            }

            if (Grid.IsRightToLeft())
            {
                GridUtil.OffsetLeft(ref clientRectangle, margin);
            }
            else
            {
                clientRectangle.Width -= margin;
            }


            if (tag != null)
            {
                string s = style.ValueMember; // Multi-column sort position is set in ValueMember (didn't want to add
                // an extra property for this and also did not want to break backward-compatibility and change Tag ...)
                int dig = (!string.IsNullOrEmpty(s) && s.Length > 1) ? 2 : 1;

                Rectangle rect;
                if (Grid.IsRightToLeft())
                {
                    rect = new Rectangle(clientRectangle.Left - margin, clientRectangle.Y, 10 * dig, clientRectangle.Height);
                }
                else
                {
                    rect = new Rectangle(clientRectangle.Right, clientRectangle.Y, 10 * dig, clientRectangle.Height);
                }

                rect = GridUtil.CenterInRect(rect, new Size(8 * dig, 8));

                //// Code used in GridTable.SetupColumnHeaderCell:
                /*
                    //// Sort Indicator
                    int index = TableDescriptor.SortedColumns.IndexOf(e.Column.MappingName);
                    SortColumnDescriptor sd = null;
                    if (index != -1)
                    {
                        sd = TableDescriptor.SortedColumns[e.Column.MappingName];
                        if (TableDescriptor.TableOptions.AllowMultiColumnSort && TableDescriptor.SortedColumns.Count > 1)
                            style.ValueMember = index.ToString();
                    }
                */

                if (s != string.Empty)
                {
                    //// Draw sort-position slightly to the right and above.
                    rect.Offset(-4 * (dig - 1), -4);
                    Font numFont = new Font(style.Font.Facename, 7, style.Font.FontStyle);
                    DrawText(g, s, numFont, rect, style, SystemColors.ControlDark, Grid.IsRightToLeft());
                    numFont.Dispose();
                    rect.Offset(4 * (dig - 1), 4);

                    if (Grid.IsRightToLeft())
                    {
                        rect.Offset(6, 0);
                    }
                    else
                    {
                        rect.Offset(-6, 0);
                    }
                }

                //// Draw Sort-Arrow
                ////Brush brush = new SolidBrush(SystemColors.ControlDark);
                Brush brush = null;
                Pen pen1 = null;

                this.Grid.Model.Options.GridVisualStylesDrawing.GetSortIconBrush(out brush, out pen1);

                int i2 = Math.Max(0, (rect.Height - 3)/2);
                rect.Inflate(-i2, -i2);
                ////Pen pen1 = new Pen(SystemColors.WindowFrame);
                GridTriangleDirection triangleDirection = listSortDirection == ListSortDirection.Ascending ? GridTriangleDirection.Up : GridTriangleDirection.Down;
                GridPaintTriangle.Paint(g, rect, triangleDirection, brush, pen1, true);
                pen1.Dispose();
                brush.Dispose();
            }
		}
		
		private void Grid_CellClick(object sender, GridCellClickEventArgs e)
		{	
			if( filterRect.IntersectsWith(new Rectangle(e.MouseEventArgs.X,e.MouseEventArgs.Y,0,0)))
			{                
                GridRangeInfo range = GridRangeInfo.Cell(e.RowIndex, e.ColIndex);
                Rectangle rec= this.Grid.RangeInfoToRectangle(range);
                Point loc = new Point(rec.Right, rec.Bottom);
				GridTableControl  grid = this.Grid as GridTableControl;                
                filterRow = e.RowIndex;
                Filtercol = e.ColIndex;                
                GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
                object[] items = GetFilterBarChoices(sty.TableCellIdentity);
                clearFilter.Text = clearFilterString +" "+ sty.TableCellIdentity.Column.HeaderText;
                
                string filterName = GetFilterName(sty);
                RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName);
                if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                    clearFilter.Enabled = true;
                else
                    clearFilter.Enabled = false;

                IList filteredValues = this.GetFilterTextList(sty);                                
                GridTableCellStyleInfo formatStyle=new GridTableCellStyleInfo();
                formatStyle.CellValueType = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.CellValueType;
                formatStyle.Format = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.Format;
                this.ddUser.SetItems(items, filteredValues, formatStyle, sty, clearFilter.Enabled);
                if (sty.GetActiveGridView() != null && sty.GetActiveGridView().GridOfficeScrollBars == Syncfusion.Windows.Forms.OfficeScrollBars.Metro)
                {
                    this.filterDialog.Font = new Font("Segoe UI", 8.25f);
                    this.filterDialog.ForeColor = Color.FromArgb(51, 51, 51);
                }
                this.filterDialog.Show(grid, loc);
				e.Cancel = true;				
			}
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Returns an array of category keys for this group and all parent groups which is used by FilterBarCells and FilterBarSummary
        /// to compare whether the conditions should be applied to this group.
        /// </summary>
        /// <param name="tableCellIdentity">Cell identifier.</param>
        /// <returns>An array of category keys.</returns>
        public object[] GetUniqueGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            return g.UniqueGroupId;
        }

        /// <summary>
        /// Returns the unique choices to be displayed in the filterbar cell.
        /// </summary>
        /// <param name="tableCellIdentity">Table cell identifier.</param>
        /// <returns>Filter bar choices.</returns>
        public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            GridGroupingControl groupGrid = ((GridEngine)tableCellIdentity.Table.Engine).GroupingControl;
            if (groupGrid.OptimizeFilterPerformance)
            {
                GridCurrentCell cc = tableCellIdentity.Table.TableDescriptor.Engine.TableControl.CurrentCell;
                if (cc.Renderer is GridNestedTableControlCellRenderer)
                {
                    GridNestedTableControlCellRenderer rend = cc.Renderer as GridNestedTableControlCellRenderer;
                    cc = rend.GetNestedCurrentCell();
                }
                cc.Lock();
                tableCellIdentity.Table.CurrentElement = tableCellIdentity.DisplayElement;
                tableCellIdentity.Table.CurrentRecordManager.Lock();
                SummaryDescriptor sd = new SummaryDescriptor(
                                tableCellIdentity.Column.Name + "FilterBarChoices",
                                tableCellIdentity.Column.MappingName,
                                new CreateSummaryDelegate(FilterBarChoicesSummary.CreateSummaryMethod));
                sd.IgnoreRecordFilterCriteria = true;
                tableCellIdentity.Table.TableDescriptor.Summaries.Add(sd);
                ITreeTableSummary[] summaries = tableCellIdentity.Table.GetSummaries();
                int summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(sd);
                FilterBarChoicesSummary filtersummary = (FilterBarChoicesSummary)summaries[summaryIndex];
                object[] values = filtersummary.Values;
                tableCellIdentity.Table.TableDescriptor.Summaries.Remove(sd);
                tableCellIdentity.Table.SummariesDirty = true;
                tableCellIdentity.Table.CurrentRecordManager.Unlock();
                cc.Unlock();
                return values;
            }
            else
            {
                int summaryIndex = -1;
                ITreeTableSummary[] summaries = tableCellIdentity.DisplayElement.ParentGroup.GetSummaries(tableCellIdentity.Table);
                if (summaries.Length.Equals(0))
                {
                    SummaryDescriptor sd = new SummaryDescriptor(
                                tableCellIdentity.Column.Name + "FilterBarChoices",
                                tableCellIdentity.Column.MappingName,
                                new CreateSummaryDelegate(FilterBarChoicesSummary.CreateSummaryMethod));
                    sd.IgnoreRecordFilterCriteria = true;
                    tableCellIdentity.Table.TableDescriptor.Summaries.Add(sd);
                    summaries = tableCellIdentity.Table.GetSummaries();
                    summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(sd);
                    tableCellIdentity.Table.TableDescriptor.Summaries.Remove(sd);
                }
                else
                {
                    summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(tableCellIdentity.Table.TableDescriptor.Summaries[tableCellIdentity.Column.Name + "FilterBarChoices"]);
                }
                if (summaryIndex != -1)
                {
                    FilterBarChoicesSummary filterBarChoicesSummary = (FilterBarChoicesSummary)summaries[summaryIndex];
                    return filterBarChoicesSummary.Values;
                }
                else
                {
                    object[] result = null;
                    GridGroupingControl gc = tableCellIdentity.Table.Engine.ParentControl;
                    GridQueryFilterBarChoicesEventArgs qe = new GridQueryFilterBarChoicesEventArgs(tableCellIdentity.Column, false, tableCellIdentity.DisplayElement);
                    gc.OnQueryFilterBarChoices(qe);
                    if (!qe.Cancel)
                    {
                        result = qe.UniqueFilterBarValues;
                    }

                    return result;
                }
            }
        }

        /// <summary>
        /// Determines the name of the filter based on the cell style information passed in.
        /// </summary>
        /// <param name="sty">cell style information.</param>
        /// <returns>Name of the filter.</returns>
        public string GetFilterName(GridTableCellStyleInfo sty)
        {
            object[] id = GetUniqueGroupId(sty.TableCellIdentity);
            string filterName = "";
            if (id == null || id.Length == 0)
            {
                filterName = sty.TableCellIdentity.Column.Name;
            }
            else
            {
                filterName = sty.TableCellIdentity.Column.Name + "@" + sty.TableCellIdentity.DisplayElement.ParentGroup.UniqueGroupIdsToString();
            }
            return filterName;
        }

        private IList GetFilterTextList(GridTableCellStyleInfo sty)
        {
            List<string> filterValus = new List<string>();
            filterValus.Clear();
            string filterName = GetFilterName(sty);
            if (!GridUtil.IsEmpty(filterName))
            {
                if (sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Contains(filterName))
                {
                    RecordFilterDescriptor fil = sty.TableCellIdentity.Table.TableDescriptor.RecordFilters[filterName];
                    foreach (FilterCondition con in fil.Conditions)
                    {
                        filterValus.Add(con.CompareText);
                    }
                }
            }
            return filterValus;
        }
        #endregion
    }

    /// <summary>
    /// Implements the DataModel part for an optimized ExcelFilterCell.
    /// </summary>
    public class GridExcelFilterCellModel : GridHeaderCellModel
    {
        /// <summary>
        /// Initializes a new <see cref="GridExcelFilterCellModel"/> object
        /// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
        /// </summary>
        /// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>
        /// <remarks>
        /// You typically access cell models through the <see cref="GridModel.CellModels"/>
        /// property of the <see cref="GridModel"/> class.
        /// </remarks>
        public GridExcelFilterCellModel(GridModel grid)
            : base(grid)
        {

        }
        /// <summary>
        /// Creates the renderer for the cell model.
        /// </summary>
        /// <param name="control">grid</param>
        /// <returns>the renderer</returns>
        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new GridExcelFilterCellRenderer(control, this);
        }
    }
    /// <summary>
    /// Implements the renderer part of an optimized ExcelFilterCell.
    /// </summary>
    public class GridExcelFilterCellRenderer : GridHeaderCellRenderer
    {
        private OptimizedFilterDropDownControl ddUser;
        private Rectangle filterRect = Rectangle.Empty;
        private bool IsMouseOver = false;
        private int colindex = -1;
        private List<string> resizedColumns = new List<string>();
        private Hashtable filterImages;
        private int filterRow = -1, Filtercol = -1;
        private ToolStripMenuItem sortAsc;
        private ToolStripMenuItem sortDesc;
        private ToolStripMenuItem clearFilter;
        private ToolStripMenuItem textFilters;
        private ToolStripMenuItem equal;
        private ToolStripMenuItem notEqual;
        private ToolStripMenuItem beginsWith;
        private ToolStripMenuItem endsWith;
        private ToolStripMenuItem contains;
        private ToolStripMenuItem customFilter;
        private int filterHeight = 16, filterWidth = 16;
        private ContextMenuStrip filterDialog;
        private string clearFilterString = SR.GetString(SR.ClearFilterFrom);

        #region Constructor

        /// <summary>
        /// Initializes a new <see cref="GridExcelFilterCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridCellModelBase"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="Grid2007ExcelFilterCellModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>
        /// References to GridControlBase and GridCellModelBase will be saved.
        /// </remarks>
        public GridExcelFilterCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            filterImages = new Hashtable();
            filterImages.Add("Filter_gray", DynamicFilterBitmaps.GetBitmap("filter_funnel_gr"));
            filterImages.Add("ClearFilter_gray", DynamicFilterBitmaps.GetBitmap("filter_set_gr"));
            filterImages.Add("Filter_white", DynamicFilterBitmaps.GetBitmap("filter_metro"));
            filterImages.Add("ClearFilter_white", DynamicFilterBitmaps.GetBitmap("filter_set_wh"));

            filterDialog = new ContextMenuStrip();

            this.ddUser = new OptimizedFilterDropDownControl();
            //hook the usercontrol save and cancel events...
            this.ddUser.UserControlSave += new EventHandler(ddUser_UserControlSave);
            this.ddUser.UserControlCancel += new EventHandler(ddUser_UserControlCancel);
            this.ddUser.Dock = DockStyle.Fill;
            sortAsc = new ToolStripMenuItem(SR.GetString(SR.SortAtoZ), DynamicFilterBitmaps.GetBitmap("sortasc"), ItemSelection_Click);
            sortAsc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortAsc);
            sortDesc = new ToolStripMenuItem(SR.GetString(SR.SortZtoA), DynamicFilterBitmaps.GetBitmap("sortdesc"), ItemSelection_Click);
            sortDesc.CheckOnClick = true;
            this.filterDialog.Items.Add(sortDesc);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            this.clearFilter = new ToolStripMenuItem(SR.GetString(SR.ClearFilterFrom), DynamicFilterBitmaps.GetBitmap("filter_delete"), ItemSelection_Click);
            this.filterDialog.Items.Add(clearFilter);
            textFilters = new ToolStripMenuItem(SR.GetString(SR.TextFilters));

            equal = new ToolStripMenuItem(SR.GetString(SR.Office2007FilterEquals), null, CustomItemSelection_Click);
            notEqual = new ToolStripMenuItem(SR.GetString(SR.CustomAutoFilterNotequal), null, CustomItemSelection_Click);
            beginsWith = new ToolStripMenuItem(SR.GetString(SR.CustomAutoFilterBeginsWith), null, CustomItemSelection_Click);
            endsWith = new ToolStripMenuItem(SR.GetString(SR.CustomAutoFilterEndsWith), null, CustomItemSelection_Click);
            contains = new ToolStripMenuItem(SR.GetString(SR.CustomAutoFilterContains), null, CustomItemSelection_Click);
            customFilter = new ToolStripMenuItem(SR.GetString(SR.CustomAutoFilter), null, CustomItemSelection_Click);
            textFilters.DropDownItems.Add(equal);
            textFilters.DropDownItems.Add(notEqual);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(beginsWith);
            textFilters.DropDownItems.Add(endsWith);
            textFilters.DropDownItems.Add(contains);
            textFilters.DropDownItems.Add(new ToolStripSeparator());
            textFilters.DropDownItems.Add(customFilter);
            this.filterDialog.Items.Add(textFilters);
            this.filterDialog.Items.Add(new ToolStripSeparator());
            ToolStripControlHost filterDropDownStrip = new ToolStripControlHost(this.ddUser);
            this.filterDialog.Items.Add(filterDropDownStrip);
            this.ddUser.BackColor = filterDropDownStrip.BackColor;
            this.Grid.CellClick += new GridCellClickEventHandler(Grid_CellClick);
            SetColumnWidths(false);
            filterLists = new Dictionary<string, List<string>>();
        }

        /// <summary>
        /// Performs filtering based on the checked items in the CheckedListBox.
        /// </summary>
        /// <param name="sender">Tool strip item</param>
        /// <param name="e">event data</param>
        private void CustomItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.textFilters.DropDownItems.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            if (index != -1)
            {
                if (this.Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                {
                    MetroCustomRowFilter textFilter = new MetroCustomRowFilter();
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        if (filterLists.ContainsKey(filterName))
                        {
                            filterLists.Remove(filterName);
                        }
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }
                else
                {
                    CustomRowFilter textFilter = new CustomRowFilter();
                    textFilter.columnName = sty.TableCellIdentity.Column.HeaderText;
                    textFilter.mappingName = sty.TableCellIdentity.Column.MappingName;
                    textFilter.SetCombo(index);
                    if (textFilter.ShowDialog() == DialogResult.OK)
                    {
                        if (filterLists.ContainsKey(filterName))
                        {
                            filterLists.Remove(filterName);
                        }
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                        RecordFilterDescriptor rfc = new RecordFilterDescriptor(filterName, textFilter.FilterString);
                        sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Add(rfc);
                    }
                }
            }
        }

        /// <summary>
        /// Performs sorting and filtering based on the ToolStrip item's selection.
        /// </summary>
        /// <param name="sender">ToolStrip item</param>
        /// <param name="e">event date</param>
        private void ItemSelection_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem selectedItem = sender as ToolStripMenuItem;
            int index = this.filterDialog.Items.IndexOf(selectedItem);
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;
            GridTableDescriptor tableDes = sty.TableCellIdentity.Table.TableDescriptor;
            string columnName = sty.TableCellIdentity.Column.Name;
            string filterName = GetFilterName(sty);
            switch (index)
            {
                case 0:
                    this.sortDesc.Checked = false;
                    if (this.sortAsc.Checked)
                    {
                        tableDes.SortedColumns.Remove(columnName);
                        tableDes.SortedColumns.Add(columnName);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 1:
                    this.sortAsc.Checked = false;
                    if (this.sortDesc.Checked)
                    {
                        tableDes.SortedColumns.Remove(columnName);
                        tableDes.SortedColumns.Add(columnName, ListSortDirection.Descending);
                    }
                    else
                    {
                        tableDes.SortedColumns.Remove(columnName);
                    }
                    break;
                case 3:
                    if (filterLists.ContainsKey(filterName))
                    {
                        filterLists.Remove(filterName);
                        sty.TableCellIdentity.Table.TableDirty = true;
                        this.Grid.Refresh();
                    }
                    sty.TableCellIdentity.Table.TableDescriptor.RecordFilters.Remove(filterName);
                    break;
            }

        }
        #endregion

        #region GridExcelFilter control events
        /// <summary>
        /// Hides the filterdialog.
        /// </summary>
        /// <param name="sender">user control</param>
        /// <param name="e">event data</param>
        private void ddUser_UserControlCancel(object sender, EventArgs e)
        {
            this.filterDialog.Hide();
        }
        /// <summary>
        /// Indicates whether the GroupingGrid is hooked with QueryRecordMeetsFilterCriteria event or not
        /// </summary>
        private bool hooked = false;

        /// <summary>
        /// Dictionary for filter choices.
        /// </summary>
        Dictionary<string, List<string>> filterLists = null;

        /// <summary>
        /// Dictionary for filter choices to be exposed.
        /// </summary>
        public Dictionary<string, List<string>> filterCollection = null;

        /// <summary>
        /// Used to get the present filtered column name.
        /// </summary>
        private string filteredColumn = String.Empty;

        private object[] oldItems = null;
		private object[] items = null;
        /// <summary>
        /// Performs filtering and hides the dialog when Ok button clicked.
        /// </summary>
        /// <param name="sender">User control</param>
        /// <param name="e">event data</param>
        private void ddUser_UserControlSave(object sender, EventArgs e)
        {
            GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;

            string filterName = GetFilterName(sty);
            if (!filterLists.ContainsKey(filterName))
            {

                filterLists.Add(filterName, ddUser.GetValues());

                filterLists[filterName].Sort();
            }
            else
            {
                filterLists[filterName] = ddUser.GetValues();
                filterLists[filterName].Sort();
            }
            List<string> list = filterLists[filterName];
            if (!hooked)
            {
                hooked = true;
                GridGroupingControl groupingGrid = ((GridTableModel)Model.Grid).Table.Engine.ParentControl as GridGroupingControl;
                groupingGrid.QueryRecordMeetsFilterCriteria += new QueryRecordMeetsFilterCriteriaEventHandler(ggc_QueryRecordMeetsFilterCriteria);
            }

            Cursor.Current = Cursors.WaitCursor;
            sty.TableCellIdentity.Table.TableDirty = true;
            this.Grid.Refresh();
            Cursor.Current = Cursors.Default;
            filterCollection = filterLists;
            filteredColumn = filterName;
            oldItems = this.items;            
            this.filterDialog.Hide();
        }
        string filterColumnName;
        /// <summary>
        /// Occurs when a record is checked whether it meets filter criteria and should appear visible in the tables DisplayElements.
        /// </summary>
        /// <param name="sender">grid</param>
        /// <param name="e">event data</param>
        private void ggc_QueryRecordMeetsFilterCriteria(object sender, QueryRecordMeetsFilterCriteriaEventArgs e)
        {
            e.Handled = true;
            e.Result = true;
            string name = e.Record.ParentChildTable.Name;
            GridTableDescriptor tableDescriptor = e.Record.ParentChildTable.ParentTableDescriptor as GridTableDescriptor;

            foreach (string filterName in filterLists.Keys)
            {
                if (tableDescriptor.Columns.Contains(filterName))
                {
                    string cellFormat = tableDescriptor.Columns[filterName].Appearance.AnyRecordFieldCell.Format;
                    string format = cellFormat.Length == 0 ? "{0}" : string.Format("{{0:{0}}}", cellFormat);
                    filterColumnName = filterName;

                    List<string> thelist = filterLists[filterName];
                    if (thelist == null || thelist.Count == 0)
                    {
                        continue;
                    }
                    string val = string.Empty;
                    DateTime recDate;
                    if (tableDescriptor.Columns[filterName].Appearance.AnyRecordFieldCell.CellValueType.Equals(typeof(DateTime)) && DateTime.TryParse(e.Record.GetValue(filterName).ToString(), out recDate))
                        val = string.Format(format, recDate);
                    else
                        val = string.Format(format, e.Record.GetValue(filterName));
                    if(val.Equals(string.Empty))
                        val = "(Blanks)";
                    int loc = thelist.BinarySearch(val);
                    e.Result = loc > -1;
                    if (!e.Result)
                    {
                        break;
                    }
                }
            }
            if (e.Result && tableDescriptor.RecordFilters.Count > 0)
                e.Handled = false; //if grid also has record filters (from custom filters) check those too.
        }


        private void SetColumnWidths(bool dispose)
        {
            this.Grid.BeginUpdate();
            GridTableControl groupingGrid = (GridTableControl)Grid;
            if (groupingGrid != null)
            {
                if (!dispose)
                {
                    GridTableDescriptor tableDesc = groupingGrid.TableDescriptor;
                    foreach (GridColumnDescriptor column in tableDesc.Columns)
                    {
                        if (!resizedColumns.Contains(column.Name))
                        {
                            this.Grid.Model.ColWidths[column.Name] += 15;
                            resizedColumns.Add(column.Name);
                        }

                    }
                }
                else
                {
                    GridTableDescriptor tableDesc = groupingGrid.TableDescriptor;
                    foreach (GridColumnDescriptor column in tableDesc.Columns)
                    {
                        if (resizedColumns.Contains(column.Name))
                        {
                            this.Grid.Model.ColWidths[column.Name] -= 15;
                            resizedColumns.Remove(column.Name);
                        }
                    }
                }
            }
            this.Grid.EndUpdate();
        }

        #endregion

        #region CellOverriden Methods
        /// <summary>
        /// Disposes the groupingGrid and its events.
        /// </summary>
        /// <param name="disposing">the boolean flag</param>
        protected override void Dispose(bool disposing)
        {
            GridGroupingControl groupingGrid = ((GridTableModel)Model.Grid).Table.Engine.ParentControl as GridGroupingControl;
            if (hooked && groupingGrid != null)
            {
                hooked = !hooked;
                groupingGrid.QueryRecordMeetsFilterCriteria -= new QueryRecordMeetsFilterCriteriaEventHandler(ggc_QueryRecordMeetsFilterCriteria);
            }

            if (Grid != null)
            {
                this.Grid.CellClick -= new GridCellClickEventHandler(Grid_CellClick);
                SetColumnWidths(true);
            }

            base.Dispose(disposing);
        }
        /// <summary>
        /// Returns the exact image of the bitmap name.
        /// </summary>
        /// <param name="bitmapName">name of the bitmap</param>
        /// <returns>The image</returns>
        private string FindImageFile(string bitmapName)
        {
            string bitmappath = "";
            for (int n = 0; n < 10; n++)
            {
                if (System.IO.File.Exists(bitmapName))
                    bitmappath = bitmapName;

                bitmapName = @"..\" + bitmapName;
            }
            return bitmappath;
        }
        /// <summary>
        /// Occurs when mouse hover enters the column header.
        /// </summary>
        /// <param name="rowIndex">row index</param>
        /// <param name="colIndex">col index</param>
        protected override void OnMouseHoverEnter(int rowIndex, int colIndex)
        {
            IsMouseOver = true;
            colindex = colIndex;
            this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
            base.OnMouseHoverEnter(rowIndex, colIndex);
        }
        /// <summary>
        /// Occurs when mouse hover leaves the column header.
        /// </summary>
        /// <param name="rowIndex">row index</param>
        /// <param name="colIndex">col index</param>
        /// <param name="e"></param>
        protected override void OnMouseHoverLeave(int rowIndex, int colIndex, EventArgs e)
        {
            IsMouseOver = false;
            colindex = colIndex;
            this.Grid.InvalidateRange(GridRangeInfo.Row(rowIndex));
            base.OnMouseHoverLeave(rowIndex, colIndex, e);
        }

        /// <summary>
        /// Occurs when drawing text in the cell
        /// </summary>
        /// <param name="g">Points to the device context.</param>
        /// <param name="textRectangle">Specifies the text rectangle</param>
        /// <param name="rowIndex">Specifies the row id.</param>
        /// <param name="colIndex">Specifies the column id.</param>
        /// <param name="style">A reference to the style object of the cell.</param>
        protected override void OnDrawDisplayText(Graphics g, Rectangle textRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            textRectangle.Width -= filterWidth + 2;
            base.OnDrawDisplayText(g, textRectangle, rowIndex, colIndex, style);
        }

       
        /// <summary>
        /// This method is called from the cell renderer's draw method to draw the contents
        /// of the client bounds for the cell.
        /// </summary>
        /// <param name="g">Points to the device context.</param>
        /// <param name="clientRectangle">Specifies the client rectangle. It is the cell rectangle without buttons and borders.</param>
        /// <param name="rowIndex">Specifies the row id.</param>
        /// <param name="colIndex">Specifies the column id.</param>
        /// <param name="style">A reference to the style object of the cell.</param>
        protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);
            Rectangle rect;
            if (Grid.IsRightToLeft())
            {
                rect = new Rectangle(clientRectangle.Left, clientRectangle.Y + (clientRectangle.Height / 6), filterWidth, filterHeight);
            }
            else
            {
                rect = new Rectangle(clientRectangle.Right - 2 * (filterWidth) + 4, clientRectangle.Y + (clientRectangle.Height / 6), filterWidth, filterHeight);
            }
            GridTableCellStyleInfo tablestyle = style as GridTableCellStyleInfo;
            Image imag_gray = (Image)filterImages["Filter_gray"];
            Image clImage_gray = (Image)filterImages["ClearFilter_gray"];
            Image imag_white = (Image)filterImages["Filter_white"];
            Image clImage_white = (Image)filterImages["ClearFilter_white"];

            if (tablestyle.TableCellIdentity.Column != null && tablestyle.TableCellIdentity.Column.AllowFilter == true)
            {
                GridTableDescriptor tableDesc = ((GridTableControl)this.Grid).TableDescriptor;
                bool isFiltered = false;
                if (filterLists.ContainsKey(tablestyle.TableCellIdentity.Column.Name)
                    /*&& GridOffice2007Filter.EnableFilteredColumnIcon*/)
                {
                    isFiltered = true;
                }

                if (IsMouseOver && colindex == colIndex)
                {
                    filterRect = rect;
                    if (!isFiltered)
                    {
                        if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                            g.DrawImage(imag_white, rect);
                        else
                            g.DrawImage(imag_gray, rect);
                    }
                   
                }
                if (GridExcelFilter.EnableFilteredColumnIcon || isFiltered)
                {
                    foreach (string filterName in filterLists.Keys)
                    {
                        if (filterName == tablestyle.TableCellIdentity.Column.Name)
                        {
                            if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                                g.DrawImage(clImage_white, rect);
                            else
                                g.DrawImage(clImage_gray, rect);
                            isFiltered = false;
                        }

                    }
                }
                else
                {
                    if (Grid.Model.Options.GridVisualStyles == GridVisualStyles.Office2010Black || Grid.Model.Options.GridVisualStyles == GridVisualStyles.Metro)
                        g.DrawImage(imag_white, rect);
                    else
                        g.DrawImage(imag_gray, rect);
                }
            }
            ShowSortIcon(g, clientRectangle, rowIndex, colIndex, style);
        }
        /// <summary>
        /// Displays the sort icon.
        /// </summary>
        /// <param name="g">Graphics</param>
        /// <param name="clientRectangle">cell rectangle</param>
        /// <param name="rowIndex">row index</param>
        /// <param name="colIndex">col index</param>
        /// <param name="style">cell style</param>
        private void ShowSortIcon(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            // No arrow needed when printing.
            object tag = style.Tag;
            if (Grid.PrintingMode || !(tag is ListSortDirection))
            {
                tag = null;
            }

            ListSortDirection listSortDirection = ListSortDirection.Ascending;
            int margin = 0;
            if (tag != null)
            {
                listSortDirection = (ListSortDirection)tag;
                margin = 12;
            }

            if (Grid.IsRightToLeft())
            {
                GridUtil.OffsetLeft(ref clientRectangle, margin);
            }
            else
            {
                clientRectangle.Width -= margin;
            }

            if (tag != null)
            {
                string s = style.ValueMember; // Multi-column sort position is set in ValueMember (didn't want to add
                // an extra property for this and also did not want to break backward-compatibility and change Tag ...)
                int dig = (!string.IsNullOrEmpty(s) && s.Length > 1) ? 2 : 1;

                Rectangle rect;
                if (Grid.IsRightToLeft())
                {
                    rect = new Rectangle(clientRectangle.Left - margin, clientRectangle.Y, 10 * dig, clientRectangle.Height);
                }
                else
                {
                    rect = new Rectangle(clientRectangle.Right, clientRectangle.Y, 10 * dig, clientRectangle.Height);
                }

                rect = GridUtil.CenterInRect(rect, new Size(8 * dig, 8));

                if (s != string.Empty)
                {
                    //// Draw sort-position slightly to the right and above.
                    rect.Offset(-4 * (dig - 1), -4);
                    Font numFont = new Font(style.Font.Facename, 7, style.Font.FontStyle);
                    DrawText(g, s, numFont, rect, style, SystemColors.ControlDark, Grid.IsRightToLeft());
                    numFont.Dispose();
                    rect.Offset(4 * (dig - 1), 4);

                    if (Grid.IsRightToLeft())
                    {
                        rect.Offset(6, 0);
                    }
                    else
                    {
                        rect.Offset(-6, 0);
                    }
                }

                //// Draw Sort-Arrow
                Brush brush = null;
                Pen pen1 = null;

                this.Grid.Model.Options.GridVisualStylesDrawing.GetSortIconBrush(out brush, out pen1);

                int i2 = Math.Max(0, (rect.Height - 3) / 2);
                rect.Inflate(-i2, -i2);

                GridTriangleDirection triangleDirection = listSortDirection == ListSortDirection.Ascending ? GridTriangleDirection.Up : GridTriangleDirection.Down;
                GridPaintTriangle.Paint(g, rect, triangleDirection, brush, pen1, true);
                pen1.Dispose();
                brush.Dispose();
            }
        }

        /// <summary>
        ///  Occurs when the user clicks inside the  cell boundary.
        /// </summary>
        /// <param name="sender">grid</param>
        /// <param name="e">event data</param>
        private void Grid_CellClick(object sender, GridCellClickEventArgs e)
        {
            if (filterRect.IntersectsWith(new Rectangle(e.MouseEventArgs.X, e.MouseEventArgs.Y, 0, 0)))
            {
                GridRangeInfo range = GridRangeInfo.Cell(e.RowIndex, e.ColIndex);
                Rectangle rec = this.Grid.RangeInfoToRectangle(range);
                Point loc = new Point(rec.Right, rec.Bottom);
                Point locn = new Point(e.MouseEventArgs.X, e.MouseEventArgs.Y);
                GridTableControl grid = this.Grid as GridTableControl;
                filterRow = e.RowIndex;
                Filtercol = e.ColIndex;
                GridTableCellStyleInfo sty = this.Grid.Model[filterRow, Filtercol] as GridTableCellStyleInfo;

                clearFilter.Text = clearFilterString + " " + sty.TableCellIdentity.Column.HeaderText;

                string filterName = GetFilterName(sty);
                if (!string.IsNullOrEmpty(filteredColumn) && filteredColumn.Equals(filterName))
                    items = oldItems;
                else
                    items = GetFilterBarChoices(sty.TableCellIdentity);

                if ((filterLists != null && filterLists.ContainsKey(filterName)) || grid.TableDescriptor.RecordFilters.Count > 0)
                    clearFilter.Enabled = true;
                else
                    clearFilter.Enabled = false;

                IList filteredValues = this.GetFilterTextList(sty);
                GridTableCellStyleInfo formatStyle = new GridTableCellStyleInfo();
                formatStyle.CellValueType = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.CellValueType;
                formatStyle.CellValue = sty.CellValue;
                formatStyle.Format = sty.TableCellIdentity.Column.Appearance.AnyRecordFieldCell.Format;
                if (sty.GetActiveGridView() != null && sty.GetActiveGridView().GridOfficeScrollBars == Syncfusion.Windows.Forms.OfficeScrollBars.Metro)
                {
                    this.filterDialog.Font = new Font("Segoe UI", 8.25f);
                    this.filterDialog.ForeColor = Color.FromArgb(51, 51, 51);
                    this.ddUser.checkedListBox1.BorderStyle = BorderStyle.FixedSingle;
                    WireButton(this.ddUser.okButton);
                    WireButton(this.ddUser.cancelButton);
                }
                this.ddUser.SetItems(items, filteredValues, formatStyle);
                this.filterDialog.Show(grid, locn);
                e.Cancel = true;
            }
        }
        /// <summary>
        /// Sets Metro button.
        /// </summary>
        /// <param name="button">button control</param>
        private void WireButton(ButtonAdv button)
        {
            button.Appearance = ButtonAppearance.Metro;
            if (button.Enabled)
                button.BackColor = Color.FromArgb(253, 143, 0);
            button.ForeColor = Color.White;
            button.Font = new Font("Segoe UI", 8.25f, FontStyle.Bold);
        }

        #endregion

        #region Helper Methods

        /// <summary>
        /// Returns an array of category keys for this group and all parent groups which is used by FilterBarCells and FilterBarSummary
        /// to compare whether the conditions should be applied to this group.
        /// </summary>
        /// <param name="tableCellIdentity">Cell identifier.</param>
        /// <returns>An array of category keys.</returns>
        public object[] GetUniqueGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            return g.UniqueGroupId;
        }

        /// <summary>
        /// Returns the unique choices to be displayed in the filterbar cell.
        /// </summary>
        /// <param name="tableCellIdentity">Table cell identifier.</param>
        /// <returns>Filter bar choices.</returns>
        public object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            GridGroupingControl groupGrid = ((GridEngine)tableCellIdentity.Table.Engine).GroupingControl;
            object[] result = null;
            GridGroupingControl gc = tableCellIdentity.Table.Engine.ParentControl;
            GridQueryFilterBarChoicesEventArgs qe = new GridQueryFilterBarChoicesEventArgs(tableCellIdentity.Column, false, tableCellIdentity.DisplayElement);
            gc.OnQueryFilterBarChoices(qe);
            if (!qe.Cancel)
            {
                result = qe.UniqueFilterBarValues;
            }
            return result;
        }

        /// <summary>
        /// Determines the name of the filter based on the cell style information passed in.
        /// </summary>
        /// <param name="sty">cell style information.</param>
        /// <returns>Name of the filter.</returns>
        public string GetFilterName(GridTableCellStyleInfo sty)
        {
            object[] id = GetUniqueGroupId(sty.TableCellIdentity);
            string filterName = "";
            if (id == null || id.Length == 0)
            {
                filterName = sty.TableCellIdentity.Column.Name;
            }
            else
            {
                filterName = sty.TableCellIdentity.Column.Name + "@" + sty.TableCellIdentity.DisplayElement.ParentGroup.UniqueGroupIdsToString();
            }
            return filterName;
        }

        /// <summary>
        /// Gets the filter values collection.
        /// </summary>
        /// <param name="sty">cell style</param>
        /// <returns>filter values</returns>
        private IList GetFilterTextList(GridTableCellStyleInfo sty)
        {
            List<string> filterValues = new List<string>();
            filterValues.Clear();
            string filterName = GetFilterName(sty);
            if (!GridUtil.IsEmpty(filterName) && filterLists.ContainsKey(filterName))
            {
                filterValues = filterLists[filterName];
            }
            return filterValues;
        }

        /// <summary>
        /// Clears all the filter choices from dictionary.
        /// </summary>
        public void ClearFilters()
        {
            filterLists.Clear();
        }
        #endregion
    }
}
